# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json

from unittest import mock
from datetime import timedelta

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from superdesk.publish.formatters.ninjs_newsroom_formatter import NewsroomNinjsFormatter
from superdesk.publish import init_app

import planning.assignments as planning_assignments
import planning.planning as planning_planning


@mock.patch("superdesk.publish.subscribers.SubscribersService.generate_sequence_number", lambda self, subscriber: 1)
class NewsroomNinjsFormatterTest(TestCase):
    def setUp(self):
        self.formatter = NewsroomNinjsFormatter()
        init_app(self.app)
        self.maxDiff = None

    def test_products(self):
        self.app.data.insert(
            "content_filters",
            [{"_id": 3, "content_filter": [{"expression": {"pf": [1], "fc": [2]}}], "name": "soccer-only3"}],
        )
        self.app.data.insert(
            "filter_conditions",
            [{"_id": 1, "field": "headline", "operator": "like", "value": "test", "name": "test-1"}],
        )
        self.app.data.insert(
            "filter_conditions", [{"_id": 2, "field": "urgency", "operator": "in", "value": "2", "name": "test-2"}]
        )
        self.app.data.insert(
            "products",
            [
                {
                    "_id": 1,
                    "content_filter": {"filter_id": 3, "filter_type": "permitting"},
                    "name": "p-1",
                    "product_type": "api",
                }
            ],
        )
        self.app.data.insert(
            "vocabularies",
            [
                {
                    "_id": "locators",
                    "display_name": "Locators",
                    "type": "unmanageable",
                    "unique_field": "qcode",
                    "items": [
                        {
                            "is_active": True,
                            "name": "NSW",
                            "qcode": "NSW",
                            "state": "New South Wales",
                            "country": "Australia",
                            "world_region": "Oceania",
                            "group": "Australia",
                        },
                    ],
                }
            ],
        )
        embargo_ts = utcnow() + timedelta(days=2)
        article = {
            "_id": "tag:aap.com.au:20150613:12345",
            "guid": "tag:aap.com.au:20150613:12345",
            "_current_version": 1,
            "anpa_category": [{"qcode": "a"}],
            "source": "AAP",
            "headline": "This is a test headline",
            "byline": "joe",
            "slugline": "slugline",
            "subject": [
                {"qcode": "02011001", "name": "international court or tribunal", "parent": None},
                {"qcode": "02011002", "name": "extradition"},
            ],
            "anpa_take_key": "take_key",
            "unique_id": "1",
            "body_html": "The story body",
            "type": "text",
            "word_count": "1",
            "priority": 1,
            "profile": "snap",
            "state": "published",
            "urgency": 2,
            "pubstatus": "usable",
            "creditline": "sample creditline",
            "keywords": ["traffic"],
            "abstract": "<p>sample <b>abstract</b></p>",
            "place": [{"name": "NSW", "qcode": "NSW"}],
            "embargo": embargo_ts,
            "body_footer": "<p>call helpline 999 if you are planning to quit smoking</p>",
            "company_codes": [{"name": "YANCOAL AUSTRALIA LIMITED", "qcode": "YAL", "security_exchange": "ASX"}],
            "genre": [{"name": "Article", "qcode": "article"}],
            "flags": {"marked_for_legal": True},
            "extra": {"foo": "test"},
            "operation": "publish",
        }
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        expected = {
            "guid": "tag:aap.com.au:20150613:12345",
            "version": "1",
            "place": [{"code": "NSW", "name": "New South Wales"}],
            "pubstatus": "usable",
            "body_html": "The story body<p>call helpline 999 if you are planning to quit smoking</p>",
            "type": "text",
            "subject": [
                {"code": "02011001", "name": "international court or tribunal"},
                {"code": "02011002", "name": "extradition"},
            ],
            "service": [{"code": "a"}],
            "source": "AAP",
            "headline": "This is a test headline",
            "byline": "joe",
            "urgency": 2,
            "priority": 1,
            "embargoed": embargo_ts.isoformat(),
            "profile": "snap",
            "slugline": "slugline",
            "description_text": "sample abstract",
            "description_html": "<p>sample <b>abstract</b></p>",
            "keywords": ["traffic"],
            "organisation": [
                {
                    "name": "YANCOAL AUSTRALIA LIMITED",
                    "rel": "Securities Identifier",
                    "symbols": [{"ticker": "YAL", "exchange": "ASX"}],
                }
            ],
            "genre": [{"name": "Article", "code": "article"}],
            "signal": [{"name": "Content Warning", "code": "cwarn", "scheme": "http://cv.iptc.org/newscodes/signal/"}],
            "extra": {"foo": "test"},
            "charcount": 67,
            "wordcount": 13,
            "readtime": 0,
            "products": [{"code": 1, "name": "p-1"}],
        }
        self.assertEqual(json.loads(doc), expected)
        article["urgency"] = 1
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        expected = {
            "guid": "tag:aap.com.au:20150613:12345",
            "version": "1",
            "place": [{"code": "NSW", "name": "New South Wales"}],
            "pubstatus": "usable",
            "body_html": "The story body<p>call helpline 999 if you are planning to quit smoking</p>",
            "type": "text",
            "subject": [
                {"code": "02011001", "name": "international court or tribunal"},
                {"code": "02011002", "name": "extradition"},
            ],
            "service": [{"code": "a"}],
            "source": "AAP",
            "headline": "This is a test headline",
            "byline": "joe",
            "urgency": 1,
            "priority": 1,
            "embargoed": embargo_ts.isoformat(),
            "profile": "snap",
            "slugline": "slugline",
            "description_text": "sample abstract",
            "description_html": "<p>sample <b>abstract</b></p>",
            "keywords": ["traffic"],
            "organisation": [
                {
                    "name": "YANCOAL AUSTRALIA LIMITED",
                    "rel": "Securities Identifier",
                    "symbols": [{"ticker": "YAL", "exchange": "ASX"}],
                }
            ],
            "genre": [{"name": "Article", "code": "article"}],
            "signal": [{"name": "Content Warning", "code": "cwarn", "scheme": "http://cv.iptc.org/newscodes/signal/"}],
            "extra": {"foo": "test"},
            "charcount": 67,
            "wordcount": 13,
            "readtime": 0,
            "products": [],
        }
        self.assertEqual(json.loads(doc), expected)

    def test_planning_data(self):
        planning_assignments.init_app(self.app)
        planning_planning.init_app(self.app)

        assignments = [{"coverage_item": "urn:coverage-id", "planning_item": "urn:planning-id"}]
        self.app.data.insert("assignments", assignments)

        article = {
            "_id": "tag:aap.com.au:20150613:12345",
            "guid": "tag:aap.com.au:20150613:12345",
            "type": "text",
            "version": 1,
            "assignment_id": assignments[0]["_id"],
        }

        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        data = json.loads(doc)

        self.assertEqual("urn:planning-id", data["planning_id"])
        self.assertEqual("urn:coverage-id", data["coverage_id"])

    def test_picture_formatter(self):
        article = {
            "guid": "20150723001158606583",
            "_current_version": 1,
            "slugline": "AMAZING PICTURE",
            "original_source": "AAP",
            "renditions": {
                "viewImage": {
                    "width": 640,
                    "href": "http://localhost:5000/api/upload/55b032041d41c8d278d21b6f/raw?_schema=http",
                    "mimetype": "image/jpeg",
                    "height": 401,
                },
                "original": {
                    "href": "https://one-api.aap.com.au/api/v3/Assets/20150723001158606583/Original/download",
                    "mimetype": "image/jpeg",
                },
            },
            "byline": "MICKEY MOUSE",
            "headline": "AMAZING PICTURE",
            "versioncreated": "2015-07-23T00:15:00.000Z",
            "ednote": "TEST ONLY",
            "type": "picture",
            "pubstatus": "usable",
            "source": "AAP",
            "description": "The most amazing picture you will ever see",
            "guid": "20150723001158606583",
            "body_footer": "<p>call helpline 999 if you are planning to quit smoking</p>",
        }
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        expected = {
            "byline": "MICKEY MOUSE",
            "renditions": {
                "original": {
                    "href": "https://one-api.aap.com.au/api/v3/Assets/20150723001158606583/Original/download",
                    "mimetype": "image/jpeg",
                },
                "viewImage": {
                    "href": "http://localhost:5000/api/upload/55b032041d41c8d278d21b6f/raw?_schema=http",
                    "mimetype": "image/jpeg",
                    "width": 640,
                    "height": 401,
                },
            },
            "headline": "AMAZING PICTURE",
            "pubstatus": "usable",
            "version": "1",
            "versioncreated": "2015-07-23T00:15:00.000Z",
            "guid": "20150723001158606583",
            "description_html": "The most amazing picture you will ever see<p>call helpline 999 if you are planning to "
            "quit smoking</p>",
            "type": "picture",
            "priority": 5,
            "slugline": "AMAZING PICTURE",
            "ednote": "TEST ONLY",
            "source": "AAP",
            "products": [],
        }
        self.assertEqual(expected, json.loads(doc))
        self.assertIn("viewImage", json.loads(doc).get("renditions"))

    def test_auto_published_item(self):
        article = {
            "guid": "foo",
            "_current_version": 1,
            "slugline": "AMAZING PICTURE",
            "original_source": "AAP",
            "byline": "MICKEY MOUSE",
            "headline": "AMAZING PICTURE",
            "versioncreated": "2015-07-23T00:15:00.000Z",
            "ednote": "TEST ONLY",
            "type": "picture",
            "pubstatus": "usable",
            "source": "AAP",
            "description": "The most amazing picture you will ever see",
            "body_footer": "<p>call helpline 999 if you are planning to quit smoking</p>",
        }
        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        processed = json.loads(doc)
        self.assertEqual(processed["guid"], "foo")
        article["ingest_id"] = "bar"
        article["ingest_version"] = "7"
        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        processed = json.loads(doc)
        self.assertEqual(processed["guid"], "foo")
        article["auto_publish"] = True
        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        processed = json.loads(doc)
        self.assertEqual(processed["guid"], "bar")
        self.assertEqual(processed["version"], "7")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from apps.content_filters.filter_condition.filter_condition_field import FilterConditionField
from apps.content_filters.filter_condition.filter_condition_value import FilterConditionValue
from apps.content_filters.filter_condition.filter_condition_operator import (
    FilterConditionOperator,
    NotInOperator,
    NotLikeOperator,
    MatchOperator,
    FilterConditionOperatorsEnum,
    ComparisonOperator,
    ExistsOperator,
)
import json


class FilterCondition:
    def __init__(self, field, operator, value):
        self.field = FilterConditionField.factory(field)
        self.operator = FilterConditionOperator.factory(operator)
        self.value = FilterConditionValue(self.operator, value)

    @staticmethod
    def parse(filter_condition):
        return FilterCondition(filter_condition["field"], filter_condition["operator"], filter_condition["value"])

    def get_mongo_query(self):
        try:
            return self.field.get_mongo_query()
        except AttributeError:
            field = self.field.get_entity_name()
            operator = self.operator.get_mongo_operator()
            value = self.value.get_value(self.field, self.operator)
            return {field: {operator: value}}

    def get_elastic_query(self):
        try:
            return self.field.get_elastic_query()
        except AttributeError:
            operator = self.operator.get_elastic_operator()
            value, field = self.value.get_elastic_value(self.field, self.operator)
            if isinstance(self.operator, MatchOperator) or isinstance(self.operator, ComparisonOperator):
                return json.loads(operator.format(field, value))
            else:
                return {operator: {field: value}}

    def contains_not(self):
        return self.operator.contains_not()

    def does_match(self, article):

        if not self.field.is_in_article(article) and type(self.operator) is not ExistsOperator:
            return (
                type(self.operator) is NotInOperator
                or type(self.operator) is NotLikeOperator
                or self.operator.operator is FilterConditionOperatorsEnum.ne
                or (
                    self.operator.operator is FilterConditionOperatorsEnum.eq
                    and self.value.value.lower() in ("no", "false", "f", "0")
                )
            )

        article_value = self.field.get_value(article)
        filter_value = self.value.get_value(self.field, self.operator)
        return self.operator.does_match(article_value, filter_value)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import requests
import re
from . import macro_replacement_fields
from decimal import Decimal
from collections import OrderedDict

RATE_SERVICE = "http://download.finance.yahoo.com/d/quotes.csv?s={}=X&f=nl1d1"


def to_currency(value, places=2, curr="", sep=",", dp=".", pos="", neg="-", trailneg=""):
    """Convert Decimal to a money formatted string.

    places:  required number of places after the decimal point
    curr:    optional currency symbol before the sign (may be blank)
    sep:     optional grouping separator (comma, period, space, or blank)
    dp:      decimal point indicator (comma or period)
             only specify as blank when places is zero
    pos:     optional sign for positive numbers: '+', space or blank
    neg:     optional sign for negative numbers: '-', '(', space or blank
    trailneg:optional trailing minus indicator:  '-', ')', space or blank

    >>> d = Decimal('-1234567.8901')
    >>> moneyfmt(d, curr='$')
    '-$1,234,567.89'
    >>> moneyfmt(d, places=0, sep='.', dp='', neg='', trailneg='-')
    '1.234.568-'
    >>> moneyfmt(d, curr='$', neg='(', trailneg=')')
    '($1,234,567.89)'
    >>> moneyfmt(Decimal(123456789), sep=' ')
    '123 456 789.00'
    >>> moneyfmt(Decimal('-0.02'), neg='<', trailneg='>')
    '<0.02>'

    """
    q = Decimal(10) ** -places  # 2 places --> '0.01'
    sign, digits, exp = value.quantize(q).as_tuple()
    result = []
    digits = list(map(str, digits))
    build, next = result.append, digits.pop
    if sign:
        build(trailneg)
    for i in range(places):
        build(next() if digits else "0")
    if places:
        build(dp)
    if not digits:
        build("0")
    i = 0
    while digits:
        build(next())
        i += 1
        if i == 3 and digits:
            i = 0
            build(sep)
    build(curr)
    build(neg if sign else pos)
    return "".join(reversed(result))


def get_rate(from_currency, to_currency):
    """Get the exchange rate."""
    r = requests.get(RATE_SERVICE.format(from_currency + to_currency), timeout=5)
    return Decimal(r.text.split(",")[1])


def format_output(original, converted):
    """Return the replacement string for the given original value."""
    if original[-1:].isalpha():
        # If there's 'm' or 'b' at the end of the original carry that across
        converted += original[-1:]
    return "{} ({})".format(original, converted)


def do_conversion(item, rate, currency, search_param, match_index, value_index):
    """Perform the conversion.

    :param item: story
    :param rate: exchange rate
    :param currency: currency symbol or prefix to be used in the results
    :param search_param: search parameter to locate the original value.  It should
    be a valid regular expression pattern, and not just an arbitrary string.
    :param match_index: int index of groups used in matching string
    :param value_index: int index of groups used in converting the value
    :return: modified story
    """
    diff = OrderedDict()

    def convert(match):
        match_item = match.group(match_index)
        value_item = match.group(value_index)
        if match_item and value_item:
            if ")" in match_item and "(" not in match_item:
                # clear any trailing parenthesis
                match_item = re.sub("[)]", "", "match_item")

            from_value = Decimal(re.sub(r"[^\d.]", "", value_item))
            precision = abs(from_value.as_tuple().exponent)
            to_value = rate * from_value
            converted_value = to_currency(to_value, places=precision, curr=currency)
            diff.setdefault(match_item, format_output(match_item, converted_value))
            return diff[match_item]

    for field in macro_replacement_fields:
        if item.get(field, None):
            item[field] = re.sub(search_param, convert, item[field])

    return (item, diff)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.metadata.item import metadata_schema
from superdesk.metadata.utils import extra_response_fields, item_url, aggregations, get_elastic_highlight_query
from eve.methods.common import resolve_document_etag
from superdesk import get_resource_service
from eve.utils import config
from flask import current_app as app
from apps.auth import get_user
from superdesk.notification import push_notification
import superdesk

SOURCE = "ingest"


class IngestResource(Resource):
    schema = {"archived": {"type": "datetime"}}
    schema.update(metadata_schema)
    extra_response_fields = extra_response_fields
    item_url = item_url
    datasource = {
        "search_backend": "elastic",
        "aggregations": aggregations,
        "es_highlight": get_elastic_highlight_query,
    }
    privileges = {"DELETE": "fetch"}


class IngestService(BaseService):
    def post_in_mongo(self, docs, **kwargs):
        for doc in docs:
            self._resolve_defaults(doc)
        self.on_create(docs)
        resolve_document_etag(docs, self.datasource)
        ids = self.backend.create_in_mongo(self.datasource, docs, **kwargs)
        self.on_created(docs)
        return ids

    def patch_in_mongo(self, id, document, original):
        res = self.backend.update_in_mongo(self.datasource, id, document, original)
        return res

    def set_ingest_provider_sequence(self, item, provider):
        """Sets the value of ingest_provider_sequence in item.

        :param item: object to which ingest_provider_sequence to be set
        :param provider: ingest_provider object, used to build the key name of sequence
        """
        sequence_number = get_resource_service("sequences").get_next_sequence_number(
            key_name="ingest_providers_{_id}".format(_id=provider[config.ID_FIELD]),
            max_seq_number=app.config["MAX_VALUE_OF_INGEST_SEQUENCE"],
        )
        item["ingest_provider_sequence"] = str(sequence_number)

    def on_deleted(self, docs):
        docs = docs if isinstance(docs, list) else [docs]
        file_ids = [
            rend.get("media")
            for doc in docs
            for rend in doc.get("renditions", {}).values()
            if not doc.get("archived") and rend.get("media")
        ]

        for file_id in file_ids:
            superdesk.app.media.delete(file_id)

        ids = [
            ref.get("residRef")
            for doc in docs
            for group in doc.get("groups", {})
            for ref in group.get("refs", {})
            if ref.get("residRef")
        ]

        if ids:
            self.delete({"_id": {"$in": ids}})

        user = get_user(required=True)
        if docs:
            push_notification("item:deleted", item=str(docs[0].get(config.ID_FIELD)), user=str(user))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
from eve.utils import ParsedRequest
from superdesk.tests import TestCase
from superdesk import get_resource_service


class ElasticSearchSettingsTest(TestCase):
    items = [
        {"_id": "123", "headline": "Test 1", "slugline": "Soccer England", "body_html": "Test"},
        {"_id": "456", "headline": "Test 2", "slugline": "Soccer England Result", "body_html": "Test"},
        {"_id": "489", "headline": "Test 3", "slugline": "Soccer-England/Result", "body_html": "Test"},
        {"_id": "321", "headline": "Test 4", "slugline": "Test Soccer England", "body_html": "Test"},
        {"_id": "654", "headline": "Test 5", "slugline": "England Test Soccer", "body_html": "Test"},
        {"_id": "984", "headline": "Test 6", "slugline": "Soccer Germany", "body_html": "Test"},
    ]

    def setUp(self):
        get_resource_service("ingest").post(self.items)

    def test_query_prefix_soccer(self):
        query = {"query": {"filtered": {"query": {"match_phrase_prefix": {"slugline.phrase": "soccer"}}}}}

        req = ParsedRequest()
        req.args = {"source": json.dumps(query)}
        query_result = get_resource_service("ingest").get(req=req, lookup=None)
        self.assertEqual(query_result.count(), 4)
        sluglines = [item.get("slugline") for item in query_result]
        self.assertIn("Soccer Germany", sluglines)
        self.assertIn("Soccer-England/Result", sluglines)
        self.assertIn("Soccer England", sluglines)
        self.assertIn("Soccer England Result", sluglines)

    def test_query_prefix_soccer_england(self):
        query = {"query": {"filtered": {"query": {"match_phrase_prefix": {"slugline.phrase": "soccer england"}}}}}

        req = ParsedRequest()
        req.args = {"source": json.dumps(query)}
        query_result = get_resource_service("ingest").get(req=req, lookup=None)
        self.assertEqual(query_result.count(), 3)
        sluglines = [item.get("slugline") for item in query_result]
        self.assertIn("Soccer-England/Result", sluglines)
        self.assertIn("Soccer England", sluglines)
        self.assertIn("Soccer England Result", sluglines)

    def test_query_prefix_soccer_england_result_without_forward_slash(self):
        query = {
            "query": {"filtered": {"query": {"match_phrase_prefix": {"slugline.phrase": "soccer-england result"}}}}
        }

        req = ParsedRequest()
        req.args = {"source": json.dumps(query)}
        query_result = get_resource_service("ingest").get(req=req, lookup=None)
        self.assertEqual(query_result.count(), 1)
        sluglines = [item.get("slugline") for item in query_result]
        self.assertIn("Soccer England Result", sluglines)

    def test_query_prefix_soccer_england_result_with_forward_slash(self):
        query = {
            "query": {"filtered": {"query": {"match_phrase_prefix": {"slugline.phrase": "soccer england/result"}}}}
        }

        req = ParsedRequest()
        req.args = {"source": json.dumps(query)}
        query_result = get_resource_service("ingest").get(req=req, lookup=None)
        self.assertEqual(query_result.count(), 1)
        sluglines = [item.get("slugline") for item in query_result]
        self.assertIn("Soccer-England/Result", sluglines)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.utils import item_url
from superdesk.auth_server.scopes import Scope
from apps.contacts.resource import ContactsResource as _ContactsResource


class ContactsResource(Resource):
    url = "contacts"
    item_url = item_url
    item_methods = ["GET"]
    resource_methods = ["GET"]
    schema = _ContactsResource.schema.copy()
    datasource = {
        "source": "contacts",
        "search_backend": "elastic",
        "default_sort": [("last_name.keyword", 1)],
        "projection": {"fields_meta": 0},
    }
    privileges = {"GET": Scope.CONTACTS_READ.name}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.activity import add_activity
from eve.utils import ParsedRequest
from flask import g
from superdesk.emails import send_user_mentioned_email
import re
import superdesk
from superdesk.notification import push_notification


def get_mentions(text):
    """Returns desk and user mentions.

    Returns the names of desks and users from a given text
    User names starts with '@' and desk names starts with '#'
    """
    user_pattern = re.compile(r"(^|\s)\@([\w.-]+)")
    desk_pattern = re.compile(r"(^|\s)\#([\w.-]+)")
    raw_user_names = set(username for match in re.finditer(user_pattern, text) for username in match.groups())
    raw_desk_names = set(deskname for match in re.finditer(desk_pattern, text) for deskname in match.groups())
    desk_names = [d.replace("_", " ") for d in raw_desk_names if d.strip()]
    user_names = [u for u in raw_user_names if u.strip()]
    return user_names, desk_names


def send_email_to_mentioned_users(doc, mentioned_users, origin):
    prefs_service = superdesk.get_resource_service("preferences")
    recipients = []
    for user in mentioned_users:
        send_email = prefs_service.email_notification_is_enabled(user_id=user)
        if send_email:
            user_doc = superdesk.get_resource_service("users").find_one(req=None, _id=user)
            recipients.append(user_doc["email"])
    if recipients:
        username = g.user.get("display_name") or g.user.get("username")
        url = "{}/#/workspace?item={}&action=edit&comments={}".format(origin, doc["item"], doc["_id"])
        send_user_mentioned_email(recipients, username, doc, url)


def get_users(user_names):
    req = ParsedRequest()
    users = superdesk.get_resource_service("users").get(req=req, lookup={"username": {"$in": user_names}})
    users = {user.get("username"): user.get("_id") for user in users}
    return users


def get_desks(desk_names):
    req = ParsedRequest()
    desks = superdesk.get_resource_service("desks").get(req=req, lookup={"name": {"$in": desk_names}})
    desks = {desk.get("name"): desk.get("_id") for desk in desks}
    return desks


def notify_mentioned_users(docs, origin, item=None):
    for doc in docs:
        mentioned_users = doc.get("mentioned_users", {}).values()
        if len(mentioned_users) > 0:
            if not item:
                item = superdesk.get_resource_service("archive").find_one(req=None, _id=doc["item"])
            add_activity(
                "user:mention",
                "",
                resource=None,
                type="comment",
                item=item,
                comment=doc.get("text"),
                comment_id=str(doc.get("_id")),
                notify=mentioned_users,
            )
            send_email_to_mentioned_users(doc, mentioned_users, origin)


def notify_mentioned_desks(docs):
    for doc in docs:
        mentioned_desks = doc.get("mentioned_desks", {}).values()
        if len(mentioned_desks) > 0:
            item = superdesk.get_resource_service("archive").find_one(req=None, _id=doc["item"])
            add_activity(
                "desk:mention",
                "",
                resource=None,
                type="comment",
                item=item,
                comment=doc.get("text"),
                comment_id=str(doc.get("_id")),
                notify_desks=mentioned_desks,
            )


def on_activity_updated(updates, original):
    if original.get("desk", "") != "":
        push_notification("desk:mention")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from content_api.companies.resource import CompaniesResource
import superdesk
from superdesk.services import BaseService


def init_app(app):
    endpoint_name = "companies"
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    CompaniesResource(endpoint_name, app=app, service=service)



import superdesk
from flask_babel import _


def init_app(app):
    superdesk.register_default_user_preference(
        "monitoring:view",
        {
            "type": "string",
            "allowed": ["list", "swimlane"],
            "view": "list",
            "default": "list",
            "label": _("Monitoring view"),
            "category": _("monitoring"),
        },
    )

    superdesk.register_default_session_preference("monitoring:view:session", None)



"""Content related helpers and utils.
"""

from eve.utils import config
from superdesk.notification import push_notification
from apps.auth import get_user


def push_content_notification(items, event="content:update"):
    """Push content:update notification for multiple items.

    It can be also 2 versions of same item in updated handler
    so that we sent event with both old and new desk/stage.

    :param list items: list of items
    :param event: custom event name
    """
    ids = {}
    desks = {}
    stages = {}
    for item in items:
        ids[str(item.get("_id", ""))] = 1
        task = item.get("task", {})
        if task.get("desk"):
            desks[str(task.get("desk", ""))] = 1
        if task.get("stage"):
            stages[str(task.get("stage", ""))] = 1
    user = get_user()
    push_notification(event, user=str(user.get(config.ID_FIELD, "")), items=ids, desks=desks, stages=stages)


def push_item_move_notification(original, doc, event="item:move"):
    """Push item:move notification.

    :param original: original doc
    :param doc: doc after updates
    :param event: event name
    """
    from_task = original.get("task", {})
    to_task = doc.get("task", {})
    user = get_user()
    push_notification(
        event,
        user=str(user.get(config.ID_FIELD, "")),
        item=str(original.get(config.ID_FIELD)),
        item_version=str(original.get(config.VERSION)),
        from_desk=str(from_task.get("desk")),
        from_stage=str(from_task.get("stage")),
        to_desk=str(to_task.get("desk")),
        to_stage=str(to_task.get("stage")),
    )


def push_expired_notification(ids, event="item:expired"):
    """Push item:expired notification.

    :param ids: list of expired item ids
    :param event: event name
    """
    push_notification(event, items={str(_id): 1 for _id in ids})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE
from superdesk.errors import ParserError
from superdesk import etree as sd_etree
from collections import OrderedDict
from superdesk import text_utils
from superdesk.utc import local_to_utc
import superdesk
import dateutil.parser
import logging

logger = logging.getLogger(__name__)
NS = {"r": "http://tempuri.org/", "a": "http://schemas.microsoft.com/2003/10/Serialization/Arrays"}


class RitzauFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse Ritzau XML feed
    """

    _subjects_map = None

    NAME = "ritzau"
    label = "Ritzau feed"
    TIMEZONE = "Europe/Copenhagen"

    def __init__(self):
        super().__init__()

        self.default_mapping = OrderedDict(
            [
                ("guid", "NewsID/text()"),
                ("body_html", {"xpath": "content/text()", "filter": sd_etree.clean_html_str}),
                (
                    "firstcreated",
                    {
                        "xpath": "PublishDate/text()",
                        "filter": self._publish_date_filter,
                    },
                ),
                (
                    "versioncreated",
                    {
                        "xpath": "PublishDate/text()",
                        "filter": self._publish_date_filter,
                    },
                ),
                ("headline", {"xpath": "headline/text()", "default": "", "key_hook": self._set_headline}),
                ("priority", {"xpath": "Priority/text()", "filter": int}),
                ("urgency", {"xpath": "Priority/text()", "filter": int}),
                ("keywords", {"xpath": "strapline/text()", "filter": lambda v: list(filter(None, v.split("/")))}),
                ("abstract", "subtitle"),
                ("byline", "origin"),
                ("version", {"xpath": "version/text()", "filter": int}),
                ("ednote", {"xpath": "TilRedaktionen/text()", "filter": self._ednote_filter}),
                ("subject", {"xpath": "IPTCList/a:int/text()", "list": True, "filter": self._subject_filter}),
            ]
        )

    @property
    def subjects_map(self):
        if self._subjects_map is None:
            voc_subjects = superdesk.get_resource_service("vocabularies").find_one(req=None, _id="subject_custom")
            if voc_subjects is not None:
                self._subjects_map = {i["qcode"]: i for i in voc_subjects["items"]}
            else:
                self._subjects_map = {}
        return self._subjects_map

    def can_parse(self, xml):
        return xml.tag.endswith("RBNews")

    def parse(self, xml, provider=None):
        item = {
            ITEM_TYPE: CONTENT_TYPE.TEXT,  # set the default type.
        }
        try:
            self.do_mapping(item, xml, namespaces=NS)
        except Exception as ex:
            raise ParserError.parseMessageError(ex, provider)
        return item

    def get_datetime(self, value):
        return dateutil.parser.parse(value)

    def _subject_filter(self, qcode):
        try:
            subject = self.subjects_map[qcode]
        except KeyError:
            return None
        else:
            if not subject.get("is_active", False):
                return None
            name = subject.get("name", "")

        return {"qcode": qcode, "name": name, "scheme": "subject_custom"}

    def _publish_date_filter(self, date_string):
        local = dateutil.parser.parse(date_string)
        return local_to_utc(self.TIMEZONE, local)

    def _set_headline(self, item, value):
        if not value:
            # if there is no headline, we use first 100 chars of body
            # cf. SDNTB-481
            value = text_utils.get_text(item.get("body_html", ""), "html")[:100]
        item["headline"] = value

    def _ednote_filter(self, ednote):
        return text_utils.get_text(ednote, lf_on_block=True).strip()


register_feed_parser(RitzauFeedParser.NAME, RitzauFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from superdesk.tests import TestCase
from superdesk.etree import etree
from superdesk.io.feed_parsers.efe_nitf import EFEFeedParser


class EFENITFTestCase(TestCase):

    filename = "efe_nitf.xml"

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture) as f:
            self.nitf = f.read()
            self.item = EFEFeedParser().parse(etree.fromstring(self.nitf), provider)

    def test_item(self):
        self.assertEqual(self.item.get("headline"), "Honduran president announces Cabinet changes")
        self.assertNotIn("byline", self.item)
        self.assertEqual(self.item["dateline"]["located"]["city"], "Tegucigalpa")
        self.assertEqual(self.item["subject"][0]["qcode"], "11006005")
        self.assertEqual(self.item["slugline"], "HONDURAS GOVERNMENT")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from .ninjs_formatter import NINJSFormatter
from superdesk.media.renditions import get_rendition_file_name
from lxml import html as lxml_html
from superdesk.etree import to_string
import logging
import re

logger = logging.getLogger(__name__)


class FTPNinjsFormatter(NINJSFormatter):
    def __init__(self):
        super().__init__()
        self.format_type = "ftp ninjs"
        self.internal_renditions = []

    def _get_source_ref(self, marker, ninjs):
        try:
            return ninjs.get("associations").get(marker).get("renditions").get("original").get("href")
        except Exception:
            logger.warning(
                "href not found for the original in FTP NINJS formatter, ensure the formatter has it enabled"
            )
            return None

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        """
        Re-wire that href's in the document to be relative to the destination FTP server root, it expects the
        destination to be an FTP server
        :param article:
        :param subscriber:
        :param recursive:
        :return:
        """

        include_original = subscriber.get("destinations")[0].get("config").get("include_original", False)
        if include_original:
            self.internal_renditions = ["original"]

        ninjs = super()._transform_to_ninjs(article, subscriber, recursive)

        # Get the path that the renditions will be pushed to
        path = subscriber.get("destinations")[0].get("config").get("associated_path")

        if path:
            renditions = ninjs.get("renditions")
            if renditions:
                for name, rendition in renditions.items():
                    rendition["href"] = (
                        "/"
                        + path.lstrip("/")
                        + ("/" if not path.endswith("/") else "")
                        + get_rendition_file_name(rendition)
                    )

        if article.get("type", "") == "text":
            # Find any embeded image references in the body_html and re-wire the img src reference and insert an id
            html_updated = False
            root_elem = lxml_html.fromstring(ninjs.get("body_html"))
            # Scan any comments for embed markers
            comments = root_elem.xpath("//comment()")
            for comment in comments:
                if "EMBED START Image" in comment.text:
                    regex = r"<!-- EMBED START Image {id: \"editor_([0:9]+)"
                    m = re.search(regex, ninjs.get("body_html", ""))
                    # Assumes the sibling of the Embed Image comment is the figure tag containing the image
                    figureElem = comment.getnext()
                    if figureElem is not None and figureElem.tag == "figure":
                        imgElem = figureElem.find("./img")
                        if imgElem is not None and m and m.group(1):
                            embed_id = "editor_" + m.group(1)
                            imgElem.attrib["id"] = embed_id
                            src = self._get_source_ref(embed_id, ninjs)
                            if src:
                                imgElem.attrib["src"] = src
                            html_updated = True
            if html_updated:
                ninjs["body_html"] = to_string(root_elem, method="html")
        return ninjs



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service, config
from superdesk.validation import ValidationError
from apps.publish.content.common import ITEM_PUBLISH
from flask_babel import lazy_gettext
import json


def validate_for_publish(item, **kwargs):
    doc = get_resource_service("archive").find_one(req=None, _id=item[config.ID_FIELD])
    validate_item = {"act": ITEM_PUBLISH, "type": doc["type"], "validate": doc}
    validation_errors = get_resource_service("validate").validate(validate_item)
    if validation_errors:
        raise ValidationError(json.dumps(validation_errors))

    return item


name = "Validate for Publish"
label = lazy_gettext("Validate for Publish")
callback = validate_for_publish
access_type = "frontend"
action_type = "direct"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .length_feet_and_inches_to_metric import feet_inches_to_metric


class FeetInchesTestCase(unittest.TestCase):
    def test_feet_to_cm(self):
        text = """His height is 5'10" today
                His height is 5'10 today
                His height is 5' 10 today
                His height is 5 ft 10 today
                His height is 5 ft. 10 today
                His height is 5 ft 10 in today
                His height is 5' 10" today
                His height is 5' today
                His height is 5 ft today
                His height is 5 feet today
                His height is 10.2-ft today
                His height is 10.2-in today
                His height is 1,020" today
                His height is 1,020 in today
                His height is 1 inch today
                His height is 1,020 inches today
                His height is 10-12 ft today
                His height is 10-12 in today
                His height is 5-foot-10 today
                His height is 5-feet-10 today"""

        item = {"body_html": text}
        res, diff = feet_inches_to_metric(item)
        self.assertEqual(diff["5'10\""], "5'10\" (1.78 m)")
        self.assertEqual(diff["5'10"], "5'10 (1.78 m)")
        self.assertEqual(diff["5' 10"], "5' 10 (1.78 m)")
        self.assertEqual(diff["5 ft 10"], "5 ft 10 (1.78 m)")
        self.assertEqual(diff["5 ft. 10"], "5 ft. 10 (1.78 m)")
        self.assertEqual(diff["5 ft 10 in"], "5 ft 10 in (1.78 m)")
        self.assertEqual(diff["5' 10\""], "5' 10\" (1.78 m)")
        self.assertEqual(diff["5'"], "5' (1.52 m)")
        self.assertEqual(diff["5 ft"], "5 ft (1.52 m)")
        self.assertEqual(diff["5 feet"], "5 feet (1.52 m)")
        self.assertEqual(diff["10.2-ft"], "10.2-ft (3.11 m)")
        self.assertEqual(diff["10.2-in"], "10.2-in (25.91 cm)")
        self.assertEqual(diff['1,020"'], '1,020" (25.91 m)')
        self.assertEqual(diff["1,020 in"], "1,020 in (25.91 m)")
        self.assertEqual(diff["1 inch"], "1 inch (2.54 cm)")
        self.assertEqual(diff["1,020 inches"], "1,020 inches (25.91 m)")
        self.assertEqual(diff["10-12 ft"], "10-12 ft (3.05-3.66 m)")
        self.assertEqual(diff["10-12 in"], "10-12 in (25.40-30.48 cm)")
        self.assertEqual(diff["5-foot-10"], "5-foot-10 (1.78 m)")
        self.assertEqual(diff["5-feet-10"], "5-feet-10 (1.78 m)")
        self.assertEqual(res["body_html"], item["body_html"])



import superdesk

from flask import current_app as app
from superdesk.utils import ListCursor
from superdesk.geonames import geonames_request, format_geoname_item


class PlacesAutocompleteResource(superdesk.Resource):

    resource_methods = ["GET"]
    item_methods = []
    schema = {
        "scheme": {"type": "string"},
        "code": {"type": "string"},
        "name": {"type": "string"},
        "state": {"type": "string"},
        "region": {"type": "string"},
        "country": {"type": "string"},
        "state_code": {"type": "string"},
        "region_code": {"type": "string"},
        "country_code": {"type": "string"},
        "continent_code": {"type": "string"},
        "feature_class": {"type": "string"},
        "location": {
            "type": "dict",
            "schema": {
                "lat": {"type": "float"},
                "lon": {"type": "float"},
            },
        },
        "tz": {"type": "string"},
    }


class PlacesAutocompleteService(superdesk.Service):
    def get(self, req, lookup):
        assert req.args.get("name"), {"name": 1}
        params = [
            ("name_startsWith", req.args.get("name")),
            ("lang", req.args.get("lang", "en").split("-")[0]),
            ("style", req.args.get("style", app.config["GEONAMES_SEARCH_STYLE"])),
        ]

        if req.args.get("featureClass"):
            params.append(("featureClass", req.args.get("featureClass")))
        else:
            for feature_class in app.config["GEONAMES_FEATURE_CLASSES"]:
                params.append(("featureClass", feature_class.upper()))

        json_data = geonames_request("search", params)
        data = [format_geoname_item(item) for item in json_data.get("geonames", [])]
        return ListCursor(data)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


def validate(item, **kwargs):
    """Updates the abstract field"""

    if not item.get("headline", "").strip():
        raise KeyError("Headline cannot be empty!")


name = "validate_headline"
label = "Validate Headline"
shortcut = "w"
callback = validate
desks = ["POLITICS"]
access_type = "backend"
action_type = "direct"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from apps.auth import get_user
from flask import request, current_app as app
from superdesk import get_resource_service, Service, config
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, metadata_schema
from superdesk.resource import Resource
from apps.archive.common import ARCHIVE, ITEM_CANCEL_CORRECTION, ITEM_CORRECTION
from superdesk.metadata.utils import item_url
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.notification import push_notification
from flask_babel import _

logger = logging.getLogger(__name__)


class ArchiveCorrectionResource(Resource):
    endpoint_name = "archive_correction"
    resource_title = endpoint_name

    schema = metadata_schema.copy()

    datasource = {
        "source": "archive",
    }
    item_url = item_url
    url = "archive/correction"
    resource_methods = []
    item_methods = ["PATCH"]
    privileges = {"PATCH": "correct"}


class ArchiveCorrectionService(Service):
    def on_update(self, updates, original):
        remove_correction = request.args.get("remove_correction") == "true"
        self._validate_correction(original)
        archive_service = get_resource_service(ARCHIVE)
        published_service = get_resource_service("published")
        archive_item = archive_service.find_one(req=None, _id=original.get(config.ID_FIELD))

        if remove_correction:
            published_article = published_service.find_one(
                req=None, guid=original.get("guid"), state=CONTENT_STATE.BEING_CORRECTED
            )

        elif original.get("state") == CONTENT_STATE.CORRECTED:
            published_article = published_service.find_one(
                req=None,
                guid=original.get("guid"),
                correction_sequence=original.get("correction_sequence"),
                state=CONTENT_STATE.CORRECTED,
            )
        else:
            published_article = published_service.find_one(req=None, guid=original.get("guid"))

        # updates for item in archive.
        if not remove_correction:
            archive_item_updates = {ITEM_STATE: CONTENT_STATE.CORRECTION, "operation": CONTENT_STATE.CORRECTION}
        elif remove_correction and archive_item.get("correction_sequence"):
            archive_item_updates = {ITEM_STATE: CONTENT_STATE.CORRECTED, "operation": ITEM_CANCEL_CORRECTION}
        else:
            archive_item_updates = {ITEM_STATE: CONTENT_STATE.PUBLISHED, "operation": ITEM_CANCEL_CORRECTION}

        # updates for item in published.
        if not remove_correction:
            published_item_updates = {
                ITEM_STATE: CONTENT_STATE.BEING_CORRECTED,
                "operation": CONTENT_STATE.BEING_CORRECTED,
            }
        elif remove_correction and published_article.get("correction_sequence"):
            published_item_updates = {ITEM_STATE: CONTENT_STATE.CORRECTED, "operation": "correct"}
        else:
            published_item_updates = {ITEM_STATE: CONTENT_STATE.PUBLISHED, "operation": ITEM_CANCEL_CORRECTION}

        # modify item in archive.
        archive_service.system_update(archive_item.get(config.ID_FIELD), archive_item_updates, archive_item)
        app.on_archive_item_updated(archive_item_updates, archive_item, ITEM_CORRECTION)

        # modify item in published.
        published_service.patch(id=published_article.get(config.ID_FIELD), updates=published_item_updates)

        user = get_user(required=True)
        push_notification("item:correction", item=original.get(config.ID_FIELD), user=str(user.get(config.ID_FIELD)))

    def _validate_correction(self, original):
        """Validates the article to be corrected.

        :param original: article to be corrected.
        :raises: SuperdeskApiError
        """
        if not original:
            raise SuperdeskApiError.notFoundError(message=_("Cannot find the article"))

        if (
            not is_workflow_state_transition_valid("correction", original[ITEM_STATE])
            and not config.ALLOW_UPDATING_SCHEDULED_ITEMS
        ):
            raise InvalidStateTransitionError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase
from superdesk.utils import SuperdeskBaseEnum


class EnumToTest(SuperdeskBaseEnum):
    red = 1
    blue = 2
    green = 3


class EnumTestCase(TestCase):
    def test_enum_from_value(self):
        self.assertEqual(EnumToTest.from_value(1), EnumToTest.red)
        self.assertIsNone(EnumToTest.from_value(4))

    def test_enum_values(self):
        values = EnumToTest.values()
        self.assertIn(1, values)
        self.assertIn(2, values)
        self.assertIn(3, values)
        self.assertNotIn(4, values)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2018-06-05 15:10

from eve.utils import config
from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "vocabularies"

    def forwards(self, mongodb_collection, mongodb_database):
        for vocabulary in mongodb_collection.find({"_id": {"$in": ["priority", "urgency"]}}):
            schema = vocabulary.get("schema", {})
            qcode = schema.get("qcode", {})
            qcode["type"] = "integer"
            schema["qcode"] = qcode
            print(mongodb_collection.update({"_id": vocabulary.get(config.ID_FIELD)}, {"$set": {"schema": schema}}))

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import os
import superdesk
import logging

from superdesk import get_resource_service


logger = logging.getLogger(__name__)


def populate_table_json(service_name, json_data):
    service = get_resource_service(service_name)
    for item in json_data:
        id_name = item.get("_id")

        if service.find_one(_id=id_name, req=None):
            service.put(id_name, item)
        else:
            service.post([item])


def process_file(filepath):
    """Insert or update the data from file.

    Filename is used to determine a collection.
    The format of the file used is JSON.
    :param filepath: absolute filepath
    :return: nothing
    """
    if not os.path.exists(filepath):
        raise FileNotFoundError

    [table_name, ext] = os.path.basename(filepath).split(".")

    with open(filepath, "rt") as vocabularies:
        json_data = json.loads(vocabularies.read())
        populate_table_json(table_name, json_data)


class AppPopulateCommand(superdesk.Command):
    """
    Insert or update data in collection using sample file.

    Specified file must be a valid json file.
    Filename will be used as a collection name where data will be inserted or updated.
    If document already exists in the collection, it will be updated.

    Example:
    ::

        $ python manage.py app:populate --filepath=data/content_types.json
        $ python manage.py app:populate --filepath=data/planning_types.json
        $ python manage.py app:populate --filepath=data/vocabularies.json

    """

    option_list = (superdesk.Option("--filepath", "-f", dest="filepath", required=True),)

    def run(self, filepath):
        process_file(filepath)


superdesk.command("app:populate", AppPopulateCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import re

from datetime import datetime

from superdesk.errors import ParserError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FileFeedParser
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, Priority, GUID_FIELD, GUID_TAG, FORMAT, FORMATS
from superdesk.utc import utc
from superdesk.metadata.utils import generate_guid


class ANPAFeedParser(FileFeedParser):
    """
    Feed Parser which can parse if the feed is in ANPA 1312 format.
    """

    NAME = "anpa1312"

    def can_parse(self, file_path):
        try:
            with open(file_path, "rb") as f:
                lines = [line for line in f]
                return re.match(b"\x16\x16\x01([a-z])([0-9]{4})\x1f([a-z0-9-]+)", lines[0], flags=re.I)
        except Exception:
            return False

    def parse(self, file_path, provider=None):
        try:
            item = {ITEM_TYPE: CONTENT_TYPE.TEXT, GUID_FIELD: generate_guid(type=GUID_TAG), FORMAT: FORMATS.HTML}

            with open(file_path, "rb") as f:
                lines = [line for line in f]

            # parse first header line
            m = re.match(b"\x16\x16\x01([a-z])([0-9]{4})\x1f([a-z-]+)", lines[0], flags=re.I)
            if m:
                item["provider_sequence"] = m.group(2).decode()

            # parse second header line
            m = re.match(
                b"([a-z]) ([a-z])(\x13|\x14)(\x11|\x12) (am-|pm-|bc-|ap-)([a-z-.0-9]+)(.*) "
                b"([0-9]{1,2})-([0-9]{1,2}) ([0-9]{0,4})",
                lines[1],
                flags=re.I,
            )
            if m:
                item["priority"] = self.map_priority(m.group(1).decode())
                item["anpa_category"] = [{"qcode": m.group(2).decode()}]
                item["slugline"] = m.group(6).decode("latin-1", "replace")
                item["anpa_take_key"] = m.group(7).decode("latin-1", "replace").strip()
                if len(m.group(10).decode()):
                    item["word_count"] = int(m.group(10).decode())
                if m.group(4) == b"\x12":
                    item[FORMAT] = FORMATS.PRESERVED

            # parse created date at the end of file
            m = re.search(b"\x03([a-z]+)-([a-z]+)-([0-9]+-[0-9]+-[0-9]+ [0-9]{2}[0-9]{2})GMT", lines[-4], flags=re.I)
            if m:
                item["firstcreated"] = datetime.strptime(m.group(3).decode(), "%m-%d-%y %H%M").replace(tzinfo=utc)
                item["versioncreated"] = item["firstcreated"]

            # parse anpa content
            body = b"".join(lines[2:])
            m = re.match(b"\x02(.*)\x03", body, flags=re.M + re.S)
            if m:
                text = m.group(1).decode("latin-1", "replace").split("\n")

                if item.get(FORMAT) == FORMATS.PRESERVED:
                    # ANPA defines a number of special characters e.g. TLI (Tab Line Inicator) Hex x08 and
                    # TTS Space Band Hex x10 These will be replaced, there will likely be others
                    body_lines = [
                        line.strip("^").replace("\b", "%08").replace("\x10", "%10")
                        for line in text
                        if line.startswith(("\t", "^", "\b"))
                    ]
                    item["body_html"] = "<pre>" + "\n".join(body_lines) + "</pre>"
                else:
                    body_lines = [line.strip() for line in text if line.startswith(("\t"))]
                    item["body_html"] = "<p>" + "</p><p>".join(body_lines) + "</p>"

                # content metadata
                header_lines = [line.strip("^<= ") for line in text if line.startswith("^")]
                if len(header_lines) > 1:
                    item["headline"] = header_lines[1].rstrip("\r\n^<= ")
                if len(header_lines) > 3:
                    item["byline"] = header_lines[-2].rstrip("\r\n^<= ")

                    # if there is no body use header lines
                    if len(body_lines) == 1 and not body_lines[0]:
                        item["body_html"] = "<p>" + "</p><p>".join(header_lines[2:]) + "</p>"

                # slugline
                if len(header_lines) > 1:
                    m = re.match("[A-Z]{2}-[A-Z]{2}--([a-z-0-9.]+)", header_lines[0], flags=re.I)
                    if m:
                        item["slugline"] = m.group(1)

                # ednote
                self._parse_ednote(header_lines, item)

            return item
        except Exception as ex:
            raise ParserError.anpaParseFileError(file_path, ex)

    def _parse_ednote(self, header_lines, item):
        for line in header_lines:
            m = re.search("EDITOR'S NOTE _(.*)", line)
            if m:
                item["ednote"] = m.group(1).strip()

    def map_priority(self, source_priority):
        mapping = {
            "f": Priority.Flash.value,
            "u": Priority.Urgent.value,
            "b": Priority.Three_Paragraph.value,
            "z": Priority.Ordinary.value,
        }

        source_priority = source_priority.lower().strip() if isinstance(source_priority, str) else ""
        return mapping.get(source_priority, Priority.Ordinary.value)


register_feed_parser(ANPAFeedParser.NAME, ANPAFeedParser())



from lxml import etree


from .base import BasePackageElement


class Preferences(BasePackageElement):
    """
    As Adobe's idml specification explains:
       ``
       The Preferences.xml file contains representations of all of the document preferences.
       ``
    """

    DOCUMENTPREFERENCE_DEFAULTS = {
        "PageHeight": "792",
        "PageWidth": "612",
        "PagesPerDocument": "1",
        "FacingPages": "true",
        "DocumentBleedTopOffset": "0",
        "DocumentBleedBottomOffset": "0",
        "DocumentBleedInsideOrLeftOffset": "0",
        "DocumentBleedOutsideOrRightOffset": "0",
        "DocumentBleedUniformSize": "true",
        "SlugTopOffset": "0",
        "SlugBottomOffset": "0",
        "SlugInsideOrLeftOffset": "0",
        "SlugRightOrOutsideOffset": "0",
        "DocumentSlugUniformSize": "false",
        "PreserveLayoutWhenShuffling": "true",
        "AllowPageShuffle": "true",
        "OverprintBlack": "true",
        "PageBinding": "LeftToRight",
        "ColumnDirection": "Horizontal",
        "Intent": "PrintIntent",
    }

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return "Resources/Preferences.xml"

    def _build_etree(self):
        self._etree = etree.Element(etree.QName(self.XMLNS_IDPKG, "Preferences"), nsmap={"idPkg": self.XMLNS_IDPKG})
        self._etree.set("DOMVersion", self.DOM_VERSION)
        self._add_documentpreference()

    def _add_documentpreference(self):
        """
        Add <DocumentPreference..> and fill it with properties.
        :return lxml.etree: DocumentPreference
        """
        # DocumentPreference
        documentpreference = etree.SubElement(
            self._etree,
            "DocumentPreference",
            attrib=self.merge_attributes(
                self.DOCUMENTPREFERENCE_DEFAULTS, self._attributes.get("DocumentPreference", {})
            ),
        )
        # Properties
        properties = etree.SubElement(documentpreference, "Properties")
        # ColumnGuideColor
        columnguidecolor = etree.SubElement(properties, "ColumnGuideColor")
        columnguidecolor.set("type", "enumeration")
        columnguidecolor.text = "Violet"
        # MarginGuideColor
        marginguidecolor = etree.SubElement(properties, "MarginGuideColor")
        marginguidecolor.set("type", "enumeration")
        marginguidecolor.text = "Magenta"

        return documentpreference



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from apps.keywords.resource import KeywordsResource
from apps.keywords.service import KeywordsService
from apps.keywords.alchemy import AlchemyKeywordsProvider


def init_app(app):
    endpoint_name = "keywords"
    service = KeywordsService(endpoint_name, backend=superdesk.get_backend())
    if app.config.get("KEYWORDS_PROVIDER") == "Alchemy":
        service.provider = AlchemyKeywordsProvider()
    KeywordsResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from .routing_rules import RoutingRuleSchemeResource, RoutingRuleSchemeService
from .rule_sets import RuleSetsService, RuleSetsResource
from superdesk import get_backend
import superdesk

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "rule_sets"
    service = RuleSetsService(endpoint_name, backend=get_backend())
    RuleSetsResource(endpoint_name, app=app, service=service)

    endpoint_name = "routing_schemes"
    service = RoutingRuleSchemeService(endpoint_name, backend=get_backend())
    RoutingRuleSchemeResource(endpoint_name, app=app, service=service)

    superdesk.privilege(
        name="rule_sets",
        label="Transformation Rules Management",
        description="User can setup transformation rules for ingest content.",
    )

    superdesk.privilege(
        name="routing_rules",
        label="Routing Rules Management",
        description="User can manage routing rules which are executed on ingested articles.",
    )



import socket
import ftplib

from contextlib import contextmanager
from flask import current_app as app

from superdesk.errors import IngestFtpError


@contextmanager
def ftp_connect(config):
    """Get ftp connection for given config.

    use with `with`

    :param config: dict with `host`, `username`, `password`, `path`, `passive` and `use_ftp`
    """
    if config.get("use_ftps", False):
        try:
            ftp = ftplib.FTP_TLS(config.get("host"), timeout=app.config.get("FTP_TIMEOUT", 300))
        except socket.gaierror as e:
            raise IngestFtpError.ftpHostError(exception=e)

        try:
            ftp.auth()
        except ftplib.error_perm as ae:
            ftp.close()
            raise IngestFtpError.ftpAuthError(exception=ae)
    else:
        try:
            ftp = ftplib.FTP(config.get("host"), timeout=app.config.get("FTP_TIMEOUT", 300))
        except socket.gaierror as e:
            raise IngestFtpError.ftpHostError(exception=e)

    if config.get("username"):
        try:
            ftp.login(config.get("username"), config.get("password"))
        except ftplib.error_perm as e:
            raise IngestFtpError.ftpAuthError(exception=e)

    # set encryption on data channel if able
    if hasattr(ftp, "prot_p"):
        ftp.prot_p()

    if config.get("path"):
        ftp.cwd(config.get("path", "").lstrip("/"))
    if config.get("passive") is False:  # only set this when not active, it's passive by default
        ftp.set_pasv(False)
    yield ftp
    ftp.close()



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2017-09-08 14:02

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "users"

    def forwards(self, mongodb_collection, mongodb_database):
        # we want all existing users to be authors by default
        print(mongodb_collection.update_many({"is_author": {"$exists": False}}, {"$set": {"is_author": True}}))

    def backwards(self, mongodb_collection, mongodb_database):
        # author was not existing before the update, so we remove the value
        print(mongodb_collection.update_many({}, {"$unset": {"is_author": ""}}))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .area_square_yards_to_metric import square_yard_to_metric


class TemperatureTestCase(unittest.TestCase):
    def test_f_to_c(self):
        text = (
            "Total area is 100.50 square yards for this land. "
            "Total area is 15.7 square yds for this land. "
            "Total area is 1 sq.yd for this land. "
            "Total area is 1-16 sq-yds for this land. "
            "Total area is 16.7-Square-yds for this land. "
            "Total area is 16,500-Square Yards. for this land. "
        )
        item = {"body_html": text}
        res, diff = square_yard_to_metric(item)
        self.assertEqual(diff["100.50 square yards"], "100.50 square yards (84.03 square meter)")
        self.assertEqual(diff["15.7 square yds"], "15.7 square yds (13.1 square meter)")
        self.assertEqual(diff["1 sq.yd"], "1 sq.yd (0.84 square meter)")
        self.assertEqual(diff["1-16 sq-yds"], "1-16 sq-yds (0.84-13 square meter)")
        self.assertEqual(diff["16.7-Square-yds"], "16.7-Square-yds (14.0 square meter)")
        self.assertEqual(diff["16,500-Square Yards"], "16,500-Square Yards (1.4 ha)")
        self.assertEqual(res["body_html"], item["body_html"])



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import logging.handlers
from superdesk.websockets_comms import SocketCommunication

logger = logging.getLogger(__name__)


def create_server(config):
    """Create websocket server and run it until it gets Ctrl+C or SIGTERM.

    :param config: config dictionary
    """
    try:
        host = config["WS_HOST"]
        port = int(config["WS_PORT"])
        broker_url = config["BROKER_URL"]
        exchange_name = config.get("WEBSOCKET_EXCHANGE")
        comms = SocketCommunication(host, port, broker_url, exchange_name)
        comms.run_server()
    except Exception:
        logger.exception("Failed to start the WebSocket server.")


if __name__ == "__main__":
    config = {"WS_HOST": "0.0.0.0", "WS_PORT": "5100", "BROKER_URL": "redis://localhost:6379"}
    create_server(config)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.errors import SuperdeskApiError
import logging
from flask import json


logger = logging.getLogger(__name__)


class UserDisabledError(SuperdeskApiError):
    """User is disabled, access restricted"""

    status_code = 403
    payload = {"is_enabled": False}
    message = "Account is disabled, access restricted."


class PasswordExpiredError(SuperdeskApiError):
    """The password of the user has expired"""

    status_code = 403
    payload = {"password_is_expired": True}
    message = "The password of the user has expired."


class CredentialsAuthError(SuperdeskApiError):
    """Credentials Not Match Auth Exception"""

    def __init__(self, credentials, message=None, error=None):
        super().__init__(status_code=401, message=message, payload={"credentials": 1})
        # pop the password so that it doesn't get logged
        credentials.pop("password", None)
        logger.warning("Login failure: %s" % json.dumps(credentials))
        if error:
            logger.error("Exception occurred: {}".format(error))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .length_miles_to_metric import miles_to_metric


class MilesTestCase(unittest.TestCase):
    def test_m_to_km(self):
        text = (
            "Tom rides 125 miles everyday. "
            "Tom rides 1 mile everyday. "
            "Tom rides 125 Miles everyday. "
            "Tom rides 125-Miles everyday. "
            "Tom rides 12.5 mi everyday. "
            "Tom rides 12.5mi everyday. "
            "Tom rides 12.5-mi everyday. "
            "Tom rides 12,500 miles everyday. "
            "Tom rides 100-12500 Miles everyday. "
        )
        item = {"body_html": text}
        res, diff = miles_to_metric(item)
        self.assertEqual(diff["125 miles"], "125 miles (201.2 km)")
        self.assertEqual(diff["1 mile"], "1 mile (1.6 km)")
        self.assertEqual(diff["125 Miles"], "125 Miles (201.2 km)")
        self.assertEqual(diff["125-Miles"], "125-Miles (201.2 km)")
        self.assertEqual(diff["12.5 mi"], "12.5 mi (20.1 km)")
        self.assertEqual(diff["12.5mi"], "12.5mi (20.1 km)")
        self.assertEqual(diff["12.5-mi"], "12.5-mi (20.1 km)")
        self.assertEqual(diff["12,500 miles"], "12,500 miles (20,117 km)")
        self.assertEqual(diff["100-12500 Miles"], "100-12500 Miles (160.9-20,117 km)")
        self.assertEqual(res["body_html"], item["body_html"])



"""
Avoid using default `language` field for text indexes
as it would throw an error when saving document with
unsupported language.

So instead we should define indexes with custom language
field (via `language_override`) and only set it when it's supported.
"""

_TEXT_MONGO_LANGUAGE = "_mongo_language"

# https://docs.mongodb.com/manual/reference/text-search-languages/#text-search-languages
_TEXT_SUPPORTED_LANGUAGES = {
    "da",
    "nl",
    "en",
    "fi",
    "fr",
    "de",
    "hu",
    "it",
    "nb",
    "pt",
    "ro",
    "ru",
    "es",
    "sv",
    "tr",
}

TEXT_INDEX_OPTIONS = {
    "background": True,
    "language_override": _TEXT_MONGO_LANGUAGE,
}


def set_mongo_lang(doc):
    """Mongo only supports certain languages and won't story document with unsupported one."""
    if doc.get("language"):
        mongo_lang = get_mongo_language(doc["language"])
        if mongo_lang:
            doc[_TEXT_MONGO_LANGUAGE] = mongo_lang


def get_mongo_language(lang):
    if not lang:
        return
    lang = lang.split("-")[0].split("_")[0]
    return lang if lang in _TEXT_SUPPORTED_LANGUAGES else None



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : GyanP
# Creation: 2020-05-13 18:03

from copy import deepcopy
from superdesk.commands.data_updates import BaseDataUpdate
from eve.utils import config


class DataUpdate(BaseDataUpdate):

    resource = "content_types"

    def forwards(self, mongodb_collection, mongodb_database):
        for content_type in mongodb_collection.find({}):
            if "schema" not in content_type:
                continue
            original_schema = deepcopy(content_type["schema"])
            for field, description in content_type["schema"].items():
                if description and description.get("mandatory_in_list"):
                    custom_fields = description.get("mandatory_in_list").get("scheme")
                    if custom_fields is not None:
                        for custom_field, custom_value in custom_fields.items():
                            # old notation
                            if custom_field == custom_value:
                                custom_fields[custom_field] = {
                                    "required": True,
                                    "readonly": False,
                                }
                            # new notation
                            elif type(custom_value) is dict:
                                custom_fields[custom_field] = {
                                    "required": custom_value.get("required", False),
                                    "readonly": custom_value.get("readonly", False),
                                }
                            # default
                            else:
                                custom_fields[custom_field] = {
                                    "required": False,
                                    "readonly": False,
                                }

            if original_schema != content_type["schema"]:
                print("update schema in content type", content_type["label"])
                mongodb_collection.update(
                    {"_id": content_type.get(config.ID_FIELD)}, {"$set": {"schema": content_type["schema"]}}
                )

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class CompanyTokenResource(Resource):
    schema = {
        "_id": {"type": "string", "unique": True},
        "expiry": {"type": "datetime"},
        "company": Resource.rel("companies", required=False),
    }

    item_url = 'regex(".+")'
    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "DELETE"]
    privileges = {"POST": "subscribers", "DELETE": "subscribers"}

    datasource = {
        "default_sort": [("_created", 1)],
    }

    mongo_prefix = MONGO_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import bcrypt
from flask import g
from apps.auth.service import AuthService
from superdesk import get_resource_service
from apps.auth.errors import CredentialsAuthError, PasswordExpiredError
from flask import current_app as app
from superdesk.utc import utcnow
import datetime


class DbAuthService(AuthService):
    def authenticate(self, credentials, ignore_expire=False):
        user = get_resource_service("auth_users").find_one(req=None, username=credentials.get("username"))
        if not user:
            raise CredentialsAuthError(credentials)

        password = credentials.get("password").encode("UTF-8")
        hashed = user.get("password").encode("UTF-8")

        if not (password and hashed):
            raise CredentialsAuthError(credentials)

        if not bcrypt.checkpw(password, hashed):
            raise CredentialsAuthError(credentials)

        if not ignore_expire and app.settings.get("PASSWORD_EXPIRY_DAYS", 0) > 0:
            days = app.settings.get("PASSWORD_EXPIRY_DAYS")
            date = user.get("password_changed_on")
            if date is None or (date + datetime.timedelta(days=days)) < utcnow():
                raise PasswordExpiredError()

        return user

    def is_authorized(self, **kwargs):
        if kwargs.get("_id") is None:
            return False

        auth = self.find_one(_id=str(kwargs.get("_id")), req=None)
        return auth and str(g.auth["_id"]) == str(auth.get("_id"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import (
    CONTENT_TYPE,
    ITEM_TYPE,
    ITEM_STATE,
    CONTENT_STATE,
    PUBLISH_SCHEDULE,
    EMBARGO,
    SCHEDULE_SETTINGS,
    ASSOCIATIONS,
)

from apps.archive.common import set_sign_off, ITEM_OPERATION
from apps.archive.archive import update_word_count
from superdesk.utc import utcnow

from .common import BasePublishService, BasePublishResource, ITEM_PUBLISH
from flask_babel import _
from flask import request

logger = logging.getLogger(__name__)


class ArchivePublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, "publish")


class ArchivePublishService(BasePublishService):
    publish_type = "publish"
    published_state = "published"
    item_operation = ITEM_PUBLISH

    def _validate(self, original, updates):
        super()._validate(original, updates)
        if original[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
            items = self.package_service.get_residrefs(original)

            if len(items) == 0 and self.publish_type == ITEM_PUBLISH:
                raise SuperdeskApiError.badRequestError(_("Empty package cannot be published!"))

    def on_update(self, updates, original):
        updates[ITEM_OPERATION] = self.item_operation
        super().on_update(updates, original)

        if not original.get("firstpublished"):
            if updates.get(SCHEDULE_SETTINGS) and updates[SCHEDULE_SETTINGS].get("utc_publish_schedule"):
                updates["firstpublished"] = updates[SCHEDULE_SETTINGS]["utc_publish_schedule"]
            else:
                updates["firstpublished"] = utcnow()

        if original.get("marked_for_user"):
            # remove marked_for_user on publish and keep it as previous_marked_user for history
            updates["previous_marked_user"] = original["marked_for_user"]
            updates["marked_for_user"] = None

        set_sign_off(updates, original)
        update_word_count(updates)
        self.set_desk(updates, original)

    def set_state(self, original, updates):
        """Set the state of the document to schedule if the publish_schedule is specified.

        :param dict original: original document
        :param dict updates: updates related to original document
        """
        updates.setdefault(ITEM_OPERATION, ITEM_PUBLISH)
        if updates.get(PUBLISH_SCHEDULE):
            updates[ITEM_STATE] = CONTENT_STATE.SCHEDULED
        elif original.get(EMBARGO) or updates.get(EMBARGO):
            updates[ITEM_STATE] = CONTENT_STATE.PUBLISHED
        else:
            super().set_state(original, updates)

    def set_desk(self, updates, original):
        if (
            not original.get("task", {}).get("desk")
            and not updates.get("task", {}).get("desk")
            and request
            and request.args
            and request.args.get("desk_id")
        ):

            desk_id = request.args["desk_id"]
            updates["task"] = updates.get("task", original.get("task", {}))
            updates["task"]["desk"] = desk_id

            if updates.get(ASSOCIATIONS):
                for key, associated_item in updates[ASSOCIATIONS].items():
                    if associated_item and not associated_item.get("task", {}).get("desk"):
                        associated_item["task"] = associated_item.get("task", {})
                        associated_item["task"]["desk"] = desk_id



import superdesk

from flask import request, current_app as app, json
from superdesk.resource import build_custom_hateoas

from apps.archive.archive import ArchiveResource, ArchiveService
from apps.archive.common import CUSTOM_HATEOAS


def elastic_filter():
    where = request.args.get("where")
    assert where, "where not set"

    params = json.loads(where)
    guid = params["guid"]
    uri = params.get("uri") or guid

    query = {
        "bool": {
            "should": [
                {"term": {"refs.uri": uri}},
                {"term": {"refs._id": guid}},
                {"term": {"refs.guid": guid}},
            ],
        },
    }

    LINKS_HOURS = app.config.get("LINKS_MAX_HOURS")
    if LINKS_HOURS:
        query["bool"].update(
            {
                "minimum_should_match": 1,
                "must": {
                    "range": {
                        "versioncreated": {
                            "gte": "now-{}h".format(int(LINKS_HOURS)),
                        },
                    },
                },
            }
        )

    return query


class LinksResource(ArchiveResource):
    item_methods = []
    resource_methods = ["GET"]
    datasource = ArchiveResource.datasource.copy()
    datasource.update(
        {
            "source": "archive",
            "elastic_filter_callback": elastic_filter,
            "elastic_filter": {"bool": {"must_not": {"term": {"version": 0}}}},
        }
    )


class LinksService(ArchiveService):
    def enhance_items(self, items):
        super().enhance_items(items)
        for item in items:
            build_custom_hateoas(CUSTOM_HATEOAS, item)

    def get(self, req, lookup):
        req.where = None  # it's handled in the elastic_filter
        return super().get(req, lookup)


def init_app(_app):
    superdesk.register_resource("links", LinksResource, LinksService, _app=_app)



"""
News resource
=============

It is an alias for archive without filtering out published items.
"""


from superdesk.resource import build_custom_hateoas
from apps.archive.archive import ArchiveResource, ArchiveService
from apps.archive.common import CUSTOM_HATEOAS


class NewsResource(ArchiveResource):
    datasource = ArchiveResource.datasource.copy()
    datasource.update(
        {
            "source": "archive",
            "elastic_filter": {"bool": {"must_not": {"term": {"version": 0}}}},
        }
    )

    resource_methods = ["GET"]
    item_methods = []


class NewsService(ArchiveService):
    def enhance_items(self, items):
        super().enhance_items(items)
        for item in items:
            build_custom_hateoas(CUSTOM_HATEOAS, item)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import copy
from superdesk import get_resource_service, config, app
from superdesk.resource import Resource


default_allowed_filters = [
    "anpa_category",
    "urgency",
    "keywords",
    "priority",
    "slugline",
    "type",
    "source",
    "headline",
    "ednote",
    "body_html",
    "genre",
    "subject",
    "desk",
    "stage",
    "sms",
    "place",
    "ingest_provider",
    "embargo",
    "featuremedia",
    "anpa_take_key",
    "agendas",
]


class FilterConditionResource(Resource):
    schema = {
        "name": {"type": "string", "required": True, "nullable": False, "empty": False, "iunique": True},
        "field": {
            "type": "string",
            "empty": False,
            "nullable": False,
            "allowed": default_allowed_filters,
        },
        "operator": {
            "type": "string",
            "allowed": [
                "in",
                "nin",
                "like",
                "notlike",
                "startswith",
                "endswith",
                "match",
                "eq",
                "ne",
                "lt",
                "lte",
                "gt",
                "gte",
                "exists",
            ],
            "empty": False,
            "nullable": False,
        },
        "value": {
            "type": "string",
            "empty": False,
            "nullable": False,
        },
    }

    additional_lookup = {"url": 'regex("[\w,.:-]+")', "field": "name"}

    privileges = {"POST": "content_filters", "PATCH": "content_filters", "DELETE": "content_filters"}

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }

    def pre_request_post(self, request):
        self._init_allowed_filters()

    def pre_request_patch(self, request, lookup):
        self._init_allowed_filters()

    def _init_allowed_filters(self):
        self.schema["field"]["allowed"] = copy.copy(default_allowed_filters)
        self.schema["field"]["allowed"].extend(app.config.get("EXCLUDED_VOCABULARY_FIELDS", []))
        lookup = {"_id": {"$nin": self.schema["field"]["allowed"]}, "type": "manageable"}
        for vocabulary in get_resource_service("vocabularies").get(req=None, lookup=lookup):
            self.schema["field"]["allowed"].append(vocabulary[config.ID_FIELD])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


class BaseComponent:
    """This is a basic interface for defining components.

    The only requirement is to implement the name method that
    uniquely identifies a component. It should also define other
    methods that implement the component functionality.
    """

    @classmethod
    def name(cls):
        raise NotImplementedError()



import superdesk
from .content_types import ContentTypesResource, ContentTypesService, CONTENT_TYPE_PRIVILEGE
from .content_types import apply_schema  # noqa


def init_app(app):
    endpoint_name = "content_types"
    service = ContentTypesService(endpoint_name, backend=superdesk.get_backend())
    ContentTypesResource(endpoint_name, app=app, service=service)
    superdesk.privilege(name=CONTENT_TYPE_PRIVILEGE, label="Content Profile", description="Manage content profiles")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.tests import TestCase
from eve.utils import ParsedRequest
from superdesk import get_resource_service
from superdesk.commands.rebuild_elastic_index import RebuildElasticIndex
from time import sleep


class RebuildIndexTestCase(TestCase):

    def test_retrieve_items_after_index_rebuilt(self):
        RESOURCE = 'ingest'
        elastic = self.app.data.elastic
        alias = elastic._resource_index(RESOURCE)
        alias_info = elastic.elastic(RESOURCE).indices.get_alias(name=alias)

        data = [{'headline': 'test {}'.format(i), 'slugline': 'rebuild {}'.format(i),
                 'type': 'text' if (i % 2 == 0) else 'picture'} for i in range(11, 21)]
        get_resource_service(RESOURCE).post(data)
        RebuildElasticIndex().run()
        alias_info_new = elastic.elastic(RESOURCE).indices.get_alias(name=alias)
        self.assertNotEqual(alias_info, alias_info_new)

        req = ParsedRequest()
        req.args = {}
        req.max_results = 25
        items = get_resource_service('ingest').get(req, {})
        self.assertEqual(10, items.count())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import blinker

__all__ = [
    "item_create",
    "item_publish",
    "item_published",
    "item_update",
    "item_updated",
    "item_fetched",
    "item_move",
    "item_moved",
    "item_rewrite",
    "item_validate",
    "item_routed",
    "item_duplicate",
    "item_duplicated",
    "archived_item_removed",
]

signals = blinker.Namespace()

#: Sent before item is created.
#:
#: .. versionadded:: 2.0
#:
#: :param sender: ArchiveService
#: :param item: item being created
item_create = signals.signal("item:create")

#: Sent before item is published.
#:
#: .. versionadded:: 1.30
#:
#: :param sender: PublishService
#: :param item: item to publish
item_publish = signals.signal("item:publish")

#: Sent when item is published.
#:
#: :param sender: PublishService
#: :param item: published item
item_published = signals.signal("item:published")

#: Sent before new version is saved.
#:
#: :param sender: ArchiveService
#: :param updates: changes to be saved
#: :param original: original item version
item_update = signals.signal("item:update")

#: Sent after new version is saved.
#:
#: .. versionadded:: 1.33
#:
#: :param sender: ArchiveService
#: :param item: updated item
#: :param original: original item version
item_updated = signals.signal("item:updated")

#: Sent after item is fetched.
#:
#: .. versionadded:: 1.29
#:
#: :param sender: FetchService
#: :param item: fetched item in production
#: :param ingest_item: item in ingest
item_fetched = signals.signal("item:fetched")

#: Sent before item is moved to different desk/stage.
#:
#: .. versionadded:: 1.33
#:
#: :param sender: MoveService
#: :param item: item after moving
#: :param original: item before moving
item_move = signals.signal("item:move")

#: Sent after item is moved to different desk/stage.
#:
#: .. versionadded:: 1.29
#:
#: :param sender: MoveService
#: :param item: item after moving
#: :param original: item before moving
item_moved = signals.signal("item:moved")


#: Sent before item update is created
#:
#: .. versionadded:: 1.29
#:
#: :param sender: ArchiveRewriteService
#: :param item: new item update
#: :param original: original item
item_rewrite = signals.signal("item:rewrite")


#: Validate item
#:
#: You can add errors to response and that will
#: prevent publishing and display those errors to users.
#:
#: .. versionadded:: 1.30.1
#:
#: :param sender: ValidateService
#: :param item: item to validate
#: :param response: human readable list or errors
#: :param error_fields: system readable errors info
item_validate = signals.signal("item:validate")


#: Sent when item is routed via internal destinations
#:
#: .. versionadded:: 1.33
#:
#: :param sender: PublishService
#: :param item: new item created via routing
#:
item_routed = signals.signal("item:routed")


#: Sent before item is duplicated
#:
#: .. versionadded:: 2.0
#:
#: :param sender: ArchiveService
#: :param item: duplicated item to be saved
#: :param original: original item
#: :param operation: operation
item_duplicate = signals.signal("item:duplicate")


#: Sent after item is duplicated
#:
#: .. versionadded:: 2.0
#:
#: :param sender: ArchiveService
#: :param item: duplicated item
#: :param original: original item
#: :param operation: operation
item_duplicated = signals.signal("item:duplicated")


#: Sent then item is removed from archived
#:
#: ..versionadded:: 1.34
#:
#: :param sender: archived service
#: :param item: item being removed from archived
archived_item_removed = signals.signal("archived:removed")


def connect(signal, subscriber):
    """Connect to signal"""
    blinker.signal(signal).connect(subscriber)


def send(signal, sender, **kwargs):
    """Send signal"""
    return blinker.signal(signal).send(sender, **kwargs)


def proxy_resource_signal(action, app):
    def handle(resource, documents):
        docs = documents
        if "_items" in documents:
            docs = documents["_items"]
        send(action, app.data, docs=docs)
        send("%s:%s" % (action, resource), app.data, docs=docs)

    return handle


def proxy_item_signal(action, app):
    def handle(resource, document):
        send(action, app.data, resource=resource, docs=[document])
        send("%s:%s" % (action, resource), app.data, docs=[document])

    return handle



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


class FormattersResource(Resource):
    """Formatters schema"""

    endpoint_name = "formatters"
    resource_methods = ["GET", "POST"]
    item_methods = []
    resource_title = endpoint_name
    schema = {
        "name": {
            "type": "string",
        },
        "article_id": {"type": "string", "required": True},
        "formatter_name": {"type": "string", "required": True},
    }
    privileges = {"POST": "archive"}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase

from superdesk.io.feed_parsers.anpa import ANPAFeedParser


def fixture(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    return os.path.normpath(os.path.join(dirname, "../fixtures", filename))


class ANPATestCase(TestCase):

    parser = ANPAFeedParser()

    def open(self, filename):
        provider = {"name": "Test"}
        return self.parser.parse(fixture(filename), provider)

    def test_open_anpa_file(self):
        item = self.open("anpa-1.tst")
        self.assertEqual("text", item["type"])
        self.assertEqual("2870", item["provider_sequence"])
        self.assertEqual(1, item["priority"])
        self.assertEqual("l", item["anpa_category"][0]["qcode"])
        self.assertEqual("text", item["type"])
        self.assertEqual(1049, item["word_count"])
        self.assertEqual("For Argentine chemo patients, mirrors can hurt", item["headline"])
        self.assertEqual("By PAUL BYRNE and ALMUDENA CALATRAVA", item["byline"])
        self.assertRegex(item["body_html"], "^<p>BUENOS")
        self.assertEqual("Argentina-Cancer", item["slugline"])
        self.assertEqual("2013-11-13T15:09:00+00:00", item["firstcreated"].isoformat())

    def test_ed_note(self):
        item = self.open("anpa-2.tst")
        self.assertEqual(
            "This is part of an Associated Press investigation into the hidden costs of green energy.", item["ednote"]
        )
        self.assertEqual("1stLd-Writethru", item["anpa_take_key"])

    def test_tab_content(self):
        item = self.open("anpa-3.tst")
        self.assertEqual("preserved", item["format"])

    def test_header_lines_only(self):
        item = self.open("anpa-4.tst")
        self.assertEqual("text", item["type"])
        self.assertEqual("HTML", item["format"])
        self.assertRegex(item["body_html"], "<p>Ex-bodyguard testifies about lewd messages sent to Paltrow")

    def test_map_priority(self):
        self.assertEqual(1, self.parser.map_priority("F"))
        self.assertEqual(2, self.parser.map_priority("U"))
        self.assertEqual(1, self.parser.map_priority("f"))
        self.assertEqual(3, self.parser.map_priority("b"))
        self.assertEqual(6, self.parser.map_priority("z"))
        self.assertEqual(6, self.parser.map_priority(None))
        self.assertEqual(6, self.parser.map_priority("dd"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
from apps.archive.archive import SOURCE as ARCHIVE
from apps.content import push_content_notification
from apps.auth import get_user_id
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.metadata.packages import RESIDREF
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.utc import utcnow
from apps.packages import PackageService
from flask_babel import _
from flask import current_app as app


package_service = PackageService()


class TranslateResource(Resource):
    endpoint_name = "translate"
    resource_title = endpoint_name

    schema = {
        "guid": {"type": "string", "required": True},
        "language": {"type": "string", "required": True},
        "desk": Resource.rel("desks", nullable=True),
    }

    url = "archive/translate"

    resource_methods = ["POST"]
    item_methods = []

    privileges = {"POST": "translate"}


class TranslateService(BaseService):
    def _translate_item(self, guid, language, task=None, service=None, state=None, **kwargs):
        if not service:
            service = ARCHIVE
        archive_service = get_resource_service(service)
        macros_service = get_resource_service("macros")
        published_service = get_resource_service("published")

        item = archive_service.find_one(req=None, guid=guid)
        if not item:
            raise SuperdeskApiError.notFoundError(_("Fail to found item with guid: {guid}").format(guid=guid))

        if not is_workflow_state_transition_valid("translate", item[ITEM_STATE]):
            raise InvalidStateTransitionError()

        if item.get("language") == language:
            return guid

        if package_service.is_package(item):
            refs = package_service.get_item_refs(item)
            for ref in refs:
                ref[RESIDREF] = self._translate_item(ref[RESIDREF], language, service=ref.get("location"), task=task)

        if not item.get("translation_id"):
            item["translation_id"] = item["guid"]

        macros_service.execute_translation_macro(item, item.get("language", None), language)

        item["language"] = language
        item["translated_from"] = guid
        item["versioncreated"] = utcnow()
        item["firstcreated"] = utcnow()
        if task:
            item["task"] = task

        extra_fields = ["translation_id", "translated_from"]

        UPDATE_TRANSLATION_METADATA_MACRO = app.config.get("UPDATE_TRANSLATION_METADATA_MACRO")

        if UPDATE_TRANSLATION_METADATA_MACRO and macros_service.get_macro_by_name(UPDATE_TRANSLATION_METADATA_MACRO):
            macros_service.execute_macro(item, UPDATE_TRANSLATION_METADATA_MACRO)

        translation_guid = archive_service.duplicate_item(
            item, extra_fields=extra_fields, state=state, operation="translate"
        )

        item.setdefault("translations", []).append(translation_guid)

        updates = {
            "translation_id": item["translation_id"],
            "translations": item["translations"],
        }

        archive_service.system_update(item["_id"], updates, item)
        published_service.update_published_items(item["_id"], "translation_id", item["_id"])
        published_service.update_published_items(item["_id"], "translations", item["translations"])

        if kwargs.get("notify", True):
            push_content_notification([item])

        return translation_guid

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            task = None
            if doc.get("desk"):
                desk = get_resource_service("desks").find_one(req=None, _id=doc["desk"]) or {}
                task = dict(desk=desk.get("_id"), stage=desk.get("working_stage"), user=get_user_id())
            ids.append(self._translate_item(doc["guid"], doc["language"], task, **kwargs))
        return ids


superdesk.workflow_action(
    name="translate",
    exclude_states=[CONTENT_STATE.SPIKED, CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED],
    privileges=["archive", "translate"],
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from decimal import Decimal
from . import unit_base
from flask_babel import lazy_gettext

SQUARE_METER_SYMBOL = "square meter"
HECTARE_SYMBOL = "ha"


def convert(square_feet, precision=1):
    """
    Converts from square feet to square m or to hectares
    :param acres: Square feet value in string
    :return: square m or to hectares value in string, and the symbol
    """
    sqf_to_sqm_rate = Decimal(0.09290304)
    symbol = SQUARE_METER_SYMBOL
    square_feet_list = square_feet.split("-")
    square_meter_list = [Decimal(a) * sqf_to_sqm_rate for a in square_feet_list]

    if any(s for s in square_meter_list if s > Decimal(10000)):
        # if the value is greater than 10000 then convert it to hectares
        square_meter_list = [unit_base.format_converted((s / Decimal(10000)), precision=1) for s in square_meter_list]
        symbol = HECTARE_SYMBOL
    else:
        square_meter_list = [unit_base.format_converted(s, precision=1) for s in square_meter_list]

    return "-".join(square_meter_list), symbol


def square_feet_to_metric(item, **kwargs):
    """Converts square feet values to metric"""

    regex = r"(\d+-?,?\.?\d*)((\s*)|(-))((sq\.?\s*-?ft)|([sS]quare\s*-?(([fF]((ee)|(oo))t)|(ft))))\b"
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = "square_feet_to_metric"
label = lazy_gettext("Area square feet to metric")
callback = square_feet_to_metric
access_type = "frontend"
action_type = "interactive"
group = lazy_gettext("area")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk

from flask import current_app as app
from superdesk.activity import add_activity, ACTIVITY_UPDATE
from superdesk.services import BaseService
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.notification import push_notification
from superdesk.utils import compare_preferences
from superdesk.resource import Resource


logger = logging.getLogger(__name__)


class RolesResource(Resource):
    schema = {
        "name": {"type": "string", "required": True, "nullable": False, "empty": False, "iunique": True},
        "description": {"type": "string"},
        "privileges": {
            "type": "dict",
            "schema": {},
            "allow_unknown": True,
        },
        "is_default": {"type": "boolean"},
        #: author role to be automatically set when user creates an item
        #: .. versionadded:: 1.29
        "author_role": {
            "type": "string",
            "nullable": True,
        },
        #: author role to be automatically set when user edits an item
        #: .. versionadded:: 1.29
        "editor_role": {
            "type": "string",
            "nullable": True,
        },
    }

    privileges = {"POST": "roles", "DELETE": "roles", "PATCH": "roles"}

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }


class RolesService(BaseService):
    def on_update(self, updates, original):
        if updates.get("is_default"):
            # if we are updating the role that is already default that is OK
            if original.get("is_default"):
                return
            self.remove_old_default()

    def on_create(self, docs):
        for doc in docs:
            # if this new one is default need to remove the old default
            if doc.get("is_default"):
                self.remove_old_default()

    def on_delete(self, docs):
        if docs.get("is_default"):
            raise SuperdeskApiError.forbiddenError("Cannot delete the default role")
        # check if there are any users in the role
        user = get_resource_service("users").find_one(req=None, role=docs.get("_id"))
        if user:
            raise SuperdeskApiError.forbiddenError("Cannot delete the role, it still has users in it!")

    def remove_old_default(self):
        # see if there is already a default role and set it to no longer default
        role_id = self.get_default_role_id()
        # make it no longer default
        if role_id:
            role = self.find_one(req=None, is_default=True)
            get_resource_service("roles").update(role_id, {"is_default": False}, role)

    def get_default_role_id(self):
        role = self.find_one(req=None, is_default=True)
        return role.get("_id") if role is not None else None

    def on_updated(self, updates, role):
        self.__send_notification(updates, role)

    def __send_notification(self, updates, role):
        role_id = role["_id"]

        role_users = superdesk.get_resource_service("users").get_users_by_role(role_id)
        notified_users = [user["_id"] for user in role_users]

        if "privileges" in updates:
            privileges_updated = False
            added, removed, modified = compare_preferences(role.get("privileges", {}), updates["privileges"])
            if len(removed) > 0 or (1, 0) in modified.values():
                push_notification("role_privileges_revoked", updated=1, role_id=str(role_id))
                privileges_updated = True
            if len(added) > 0 or (0, 1) in modified.values():
                activity = add_activity(
                    ACTIVITY_UPDATE,
                    "role {{role}} has been granted new privileges: Please re-login.",
                    self.datasource,
                    notify=notified_users,
                    can_push_notification=False,
                    role=role.get("name"),
                )
                push_notification("activity", _dest=activity["recipients"])
                privileges_updated = True
            if privileges_updated:
                app.on_role_privileges_updated(role, role_users)

        else:
            push_notification("role", updated=1, user_id=str(role_id))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk

from bson import ObjectId
from flask import current_app as app
from superdesk import get_resource_service, config
from superdesk.utc import utcnow
from superdesk.errors import SubscriberError, SuperdeskPublishError, PublishQueueError

logger = logging.getLogger(__name__)
extensions = {"NITF": "ntf", "XML": "xml", "NINJS": "json"}


class PublishServiceBase:
    """Base publish service class."""

    NAME = None
    CONFIG = None
    DEFAULT_EXT = "txt"

    def _transmit(self, queue_item, subscriber):
        """Performs the publishing of the queued item. Implement in subclass

        @param queue_item: the queued item document
        @type queue_item: dict
        @param subscriber: the subscriber document
        @type subscriber: dict
        """
        raise NotImplementedError()

    def _transmit_media(self, media, destination):
        """Transmit media file. Implement in subclass"""
        raise NotImplementedError()

    def transmit(self, queue_item):
        subscriber = get_resource_service("subscribers").find_one(req=None, _id=queue_item["subscriber_id"])

        if not subscriber.get("is_active"):
            raise SubscriberError.subscriber_inactive_error(Exception("Subscriber inactive"), subscriber)
        else:
            try:
                # "formatted_item" is the item as str
                # "encoded_item" is the bytes version
                # if "encoded_item_id" exists we use it, else
                # we fill encoded_item using "formatted_item" and "item_encoding"
                if "encoded_item_id" in queue_item:
                    encoded_item_id = queue_item["encoded_item_id"]
                    queue_item["encoded_item"] = app.storage.get(encoded_item_id).read()
                else:
                    encoding = queue_item.get("item_encoding", "utf-8")
                    queue_item["encoded_item"] = queue_item["formatted_item"].encode(encoding, errors="replace")
                self._transmit(queue_item, subscriber) or []
                self.update_item_status(queue_item, "success")
            except SuperdeskPublishError as error:
                self.update_item_status(queue_item, "error", error)
                self.close_transmitter(subscriber, error)
                raise error

    def transmit_media(self, media, subscriber=None, destination=None):
        if subscriber and destination and subscriber.get("is_active"):
            return self._transmit_media(media, destination)

    def close_transmitter(self, subscriber, error):
        """
        Checks if the transmitter has the error code set in the list of critical errors then closes the transmitter.

        :param error: The error thrown during transmission
        """

        if subscriber.get("critical_errors", {}).get(str(error.code)):
            update = {
                "is_active": False,
                "last_closed": {
                    "closed_at": utcnow(),
                    "message": "Subscriber made inactive due to critical error: {}".format(error),
                },
            }

            get_resource_service("subscribers").system_update(subscriber[config.ID_FIELD], update, subscriber)

    def update_item_status(self, queue_item, status, error=None):
        try:
            item_update = {"state": status}
            if status == "in-progress":
                item_update["transmit_started_at"] = utcnow()
            elif status == "success":
                item_update["completed_at"] = utcnow()
            elif status == "error" and error:
                item_update["error_message"] = "{}:{}".format(error, str(error.system_exception))

            publish_queue_service = superdesk.get_resource_service("publish_queue")
            queue_id = (
                ObjectId(queue_item.get("_id")) if isinstance(queue_item.get("_id"), str) else queue_item.get("_id")
            )
            publish_queue_service.patch(queue_id, item_update)
        except Exception as ex:
            raise PublishQueueError.item_update_error(ex)

    @classmethod
    def get_file_extension(cls, queue_item):
        try:
            format_ = queue_item["destination"]["format"].upper()
        except KeyError:
            pass
        else:
            # "in" is used in addition of equality, so subclass can inherit extensions
            # e.g.: "NITF" will work for "NTB NITF"
            try:
                return extensions[format_]
            except KeyError:
                for f, ext in extensions.items():
                    if f in format_:
                        return ext
        # nothing found, we return default extension
        return cls.DEFAULT_EXT

    @classmethod
    def get_filename(cls, queue_item):
        config = queue_item.get("destination", {}).get("config", {})
        # use the file extension from config if it is set otherwise use extension for the format
        extension = config.get("file_extension") or cls.get_file_extension(queue_item)

        return "{}-{}-{}.{}".format(
            queue_item["item_id"],
            str(queue_item.get("item_version", "")),
            str(queue_item.get("published_seq_num", "")),
            extension,
        ).replace(":", "-")

    @staticmethod
    def register_file_extension(format_, ext):
        """Register new file extension

        :param format_: item format
        :param ext: extension to use
        """
        if format_ in extensions:
            logger.warning("overriding existing extension for {}".format(format_))
        extensions[format_.upper()] = ext


PublishService = PublishServiceBase


def get_publish_service():
    return PublishService


def set_publish_service(publish_service_class):
    global PublishService
    PublishService = publish_service_class



import superdesk

from flask import current_app as app, json
from superdesk.utils import get_random_string
from superdesk.media.media_operations import crop_image, process_image, encode_metadata
from apps.search_providers.proxy import PROXY_ENDPOINT
from superdesk.media.image import fix_orientation


def get_file(rendition, item):
    if item.get("fetch_endpoint"):
        if item["fetch_endpoint"] == PROXY_ENDPOINT:  # it requires provider info
            return superdesk.get_resource_service(item["fetch_endpoint"]).fetch_rendition(rendition, item=item)
        return superdesk.get_resource_service(item["fetch_endpoint"]).fetch_rendition(rendition)
    else:
        return app.media.fetch_rendition(rendition)


def get_crop_size(crop, width=800, height=600):
    """In case width or height is missing it will do the math.

    :param size: size dict with `width` or `height`
    :param crop: crop specs
    :param width: original width
    :param height: original height
    """
    if not ("CropRight" in crop and "CropLeft" in crop and "CropBottom" in crop and "CropTop" in crop):
        crop["CropRight"] = width
        crop["CropLeft"] = 0
        crop["CropBottom"] = height
        crop["CropTop"] = 0

        return {"width": width, "height": height}

    crop_width = crop["CropRight"] - crop["CropLeft"]
    crop_height = crop["CropBottom"] - crop["CropTop"]

    size = {"width": crop.get("width", crop_width), "height": crop.get("height", crop_height)}

    crop_ratio = crop_width / crop_height
    size_ratio = size["width"] / size["height"]

    # Keep crop data proportional to the size provided
    # i.e. if the rendition is 4x3, make sure the crop data is also a 4x3 aspect ratio
    # but make sure it won't exceed picture boundaries
    if round(crop_ratio, 2) > round(size_ratio, 2):
        crop_width = int(crop_height * size_ratio)
        crop["CropRight"] = crop["CropLeft"] + crop_width
    elif round(crop_ratio, 2) < round(size_ratio, 2):
        crop_height = int(crop_width / size_ratio)
        crop["CropBottom"] = crop["CropTop"] + crop_height

    return size


class PictureCropService(superdesk.Service):
    """Crop original image of picture item and return its url.

    It is used for embedded images within text item body.
    """

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            item = doc.pop("item")
            crop = doc.pop("crop")
            orig = item["renditions"]["original"]
            size = get_crop_size(crop, orig.get("width", 800), orig.get("height", 600))
            orig_file = get_file(orig, item)
            filename = get_random_string()
            ok, output = crop_image(orig_file, filename, crop, size)
            if ok:
                metadata = encode_metadata(process_image(orig_file))
                metadata.update({"length": json.dumps(len(output.getvalue()))})
                output = fix_orientation(output)
                media = app.media.put(output, filename, orig["mimetype"], metadata=metadata)
                doc["href"] = app.media.url_for_media(media, orig["mimetype"])
                doc["width"] = output.width
                doc["height"] = output.height
                doc["metadata"] = metadata
                ids.append(media)
        return ids


class PictureCropResource(superdesk.Resource):

    item_methods = []
    resource_methods = ["POST"]
    privileges = {"POST": "archive"}

    schema = {
        "item": {"type": "dict", "required": True, "empty": False},
        "crop": {"type": "dict", "required": True, "empty": False},
        "href": {"type": "string", "readonly": True},
        "width": {"type": "integer", "readonly": True},
        "height": {"type": "integer", "readonly": True},
    }


def init_app(app):
    superdesk.register_resource("picture_crop", PictureCropResource, PictureCropService, "archive")



#!/usr/bin/env python3
#
# This file is part of Superdesk.
#
# Copyright 2013-2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from setuptools import setup, find_packages

LONG_DESCRIPTION = "Superdesk Server Core"

install_requires = [
    # temporary requirement to get urllib in a version compatible with requests
    # to be kept until requests update its requirements
    # (cf. https://github.com/psf/requests/issues/5654
    # and https://github.com/psf/requests/pull/5651)
    "urllib3<1.26",
    "eve==1.1.2",
    "eve-elastic>=7.0.0,<8.0.0",
    "flask>=1.1,<1.2",
    "flask-mail>=0.9,<0.10",
    "flask-script>=2.0.5,<3.0",
    "flask-babel>=1.0,<1.1",
    "pillow>=8.0,<9.0",
    "arrow>=0.4,<=0.13",
    "bcrypt>=3.1.1,<3.2",
    "blinker>=1.3,<1.5",
    "celery[redis]>=4.3.0,<4.4",
    "cerberus>=1.3.2,<1.4",
    "redis>=3.2.0,<3.3",
    "kombu>=4.5,<4.6",
    "feedparser>=5.2,<5.3",
    "hachoir<=3.0a3",
    "HermesCache>=0.6.0,<0.8.0",
    "python-magic>=0.4,<0.5",
    "ldap3>=2.2.4,<2.6",
    "pytz>=2015.4",
    "tzlocal>=1.2.2,<2.0",
    "raven[flask]>=5.10,<7.0",
    "requests>=2.7.0,<3.0",
    "boto3>=1.1.4,<1.6",
    "websockets>=3.0,<7.0",
    "mongolock>=1.3.4,<1.4",
    "PyYAML>=4.2b1,<5.0",
    "lxml>=3.8,<4.2",
    "python-twitter==3.5",
    "chardet<4.0",
    "pymongo>=3.8,<3.12",
    "croniter<0.4",
    "python-dateutil<2.8",
    "unidecode==0.04.21",
    "authlib>0.14,<0.15",
    "draftjs-exporter[lxml]<2.2",
    "werkzeug>=1.0,<1.1",
    "regex==2020.7.14",
    "flask-oidc-ex==0.5.5",
    "vine==1.3.0",
    # to be replaced by stdlib version when we use Python 3.8+
    "importlib_metadata<3.2",
]

package_data = {
    "superdesk": [
        "templates/*.txt",
        "templates/*.html",
        "locators/data/*.json",
        "io/data/*.json",
        "data_updates/*.py",
        "data_updates/*.js",
        "translations/*.po",
        "translations/*.mo",
    ],
    "apps": [
        "prepopulate/*.json",
        "prepopulate/data_init/*.json",
        "io/data/*.json",
    ],
}

setup(
    name="Superdesk-Core",
    version="2.1.1",
    description="Superdesk Core library",
    long_description=LONG_DESCRIPTION,
    author="petr jasek",
    author_email="petr.jasek@sourcefabric.org",
    url="https://github.com/superdesk/superdesk-core",
    license="GPLv3",
    platforms=["any"],
    packages=find_packages(exclude=["tests", "features"]),
    package_data=package_data,
    include_package_data=True,
    setup_requires=["setuptools_scm"],
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive.common import ARCHIVE
from superdesk import get_resource_service
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, PUBLISH_STATES, ITEM_STATE
from superdesk.resource import Resource

from apps.packages.package_service import PackageService, create_root_group, get_item_ref
from eve.utils import config
from superdesk.validation import ValidationError
from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService
from superdesk.metadata.packages import GROUPS, GROUP_ID, REFS, RESIDREF, ROOT_GROUP, ID_REF, PACKAGE_TYPE
from flask_babel import _


class PublishedPackageItemsResource(Resource):
    schema = {
        "package_id": {"type": "string", "required": True},
        "new_items": {
            "type": "list",
            "required": True,
            "schema": {"type": "dict", "schema": {"group": {"type": "string"}, "item_id": {"type": "string"}}},
        },
    }
    datasource = {"source": "archive"}
    resource_methods = ["POST"]
    privileges = {"POST": ARCHIVE}


class PublishedPackageItemsService(BaseService):
    package_service = PackageService()

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            original = get_resource_service(ARCHIVE).find_one(req=None, _id=doc["package_id"])
            if not original or original[ITEM_TYPE] != CONTENT_TYPE.COMPOSITE:
                raise SuperdeskApiError.badRequestError(_("Invalid package identifier"))
            if original[ITEM_STATE] not in PUBLISH_STATES:
                raise SuperdeskApiError.badRequestError(_("Package was not published"))

            items = {}
            for new_item in doc["new_items"]:
                item = get_resource_service(ARCHIVE).find_one(req=None, _id=new_item["item_id"])
                if not item:
                    raise SuperdeskApiError.badRequestError(
                        _("Invalid item identifier  {item_id}").format(item_id=new_item["item_id"])
                    )
                try:
                    self.package_service.check_for_circular_reference(original, new_item["item_id"])
                except ValidationError:
                    raise SuperdeskApiError.badRequestError(
                        _("Circular reference in item {item_id}").format(item_id=new_item["item_id"])
                    )
                items[item[config.ID_FIELD]] = item

            updates = {key: original[key] for key in [config.ID_FIELD, PACKAGE_TYPE, GROUPS] if key in original}
            create_root_group([updates])
            items_refs = []
            for new_item in doc["new_items"]:
                items_refs.append(self._set_item_assoc(updates, new_item, items[new_item["item_id"]]))
            get_resource_service(ARCHIVE).system_update(original[config.ID_FIELD], updates, original)
            for item_ref in items_refs:
                self.package_service.update_link(updates, item_ref)

            items_published = [new_item[ITEM_STATE] in PUBLISH_STATES for new_item in items.values()]
            if any(items_published):
                get_resource_service("archive_correct").patch(id=doc["package_id"], updates=updates)

            ids.append(original[config.ID_FIELD])
        return ids

    def _set_item_assoc(self, package, new_item, item_doc):
        group = self._get_group(package, new_item["group"])
        for assoc in group[REFS]:
            if assoc.get(RESIDREF) == new_item["item_id"]:
                return assoc
        item_ref = get_item_ref(item_doc)
        group[REFS].append(item_ref)
        return item_ref

    def _get_group(self, package, group):
        for package_group in package[GROUPS]:
            if group == package_group[GROUP_ID]:
                return package_group
        self._add_group_in_root(group, package[GROUPS])
        package[GROUPS].append({GROUP_ID: group, REFS: []})
        return package[GROUPS][-1]

    def _add_group_in_root(self, group, groups):
        root_refs = []
        for group_meta in groups:
            if group_meta.get(GROUP_ID) == ROOT_GROUP:
                root_refs = [ref[ID_REF] for ref in group_meta[REFS]]
                if group not in root_refs:
                    group_meta[REFS].append({ID_REF: group})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from superdesk.resource import Resource
import superdesk
from content_api import MONGO_PREFIX


logger = logging.getLogger(__name__)


class AuthUsersResource(Resource):
    """This resource is for authentication only.

    On users `find_one` never returns a password due to the projection.
    """

    datasource = {"source": "users"}
    schema = {
        "email": {
            "type": "string",
        },
        "password": {
            "type": "string",
        },
        "is_enabled": {"type": "boolean"},
        "is_approved": {"type": "boolean"},
    }
    item_methods = []
    resource_methods = []
    internal_resource = True
    mongo_prefix = MONGO_PREFIX


class AuthResource(Resource):
    schema = {
        "email": {"type": "string", "required": True},
        "password": {"type": "string", "required": True},
        "token": {"type": "string"},
        "user": Resource.rel("users", True),
    }
    resource_methods = ["POST"]
    item_methods = ["GET", "DELETE"]
    public_methods = ["POST", "DELETE"]
    extra_response_fields = ["user", "token", "username"]
    datasource = {"source": "auth"}


superdesk.intrinsic_privilege("auth", method=["DELETE"])



from lxml import etree

from .story import Story


class StoryList(Story):
    """
    Story which represents `ul`, `ol` html tags.
    """

    def _add_story(self):
        # merge Story attributes
        story_attributes = self.merge_attributes(self.STORY_DEFAULTS, self._attributes.get("Story", {}))
        story_attributes.update({"Self": self.self_id})
        # Story
        story = etree.SubElement(self._etree, "Story", attrib=story_attributes)
        # StoryPreference
        etree.SubElement(
            story,
            "StoryPreference",
            attrib=self.merge_attributes(self.STORYPREFERENCE_DEFAULTS, self._attributes.get("StoryPreference", {})),
        )

        if self._markup_tag:
            # XMLElement to tag a story
            paragraphstylerange_container = etree.SubElement(
                story,
                "XMLElement",
                attrib={
                    "Self": "{}_{}".format(self.self_id, self._markup_tag.lower()),
                    "XMLContent": self.self_id,
                    "MarkupTag": "XMLTag/{}".format(self._markup_tag),
                },
            )
        else:
            paragraphstylerange_container = story

        # ParagraphStyleRange
        paragraphstylerange = etree.SubElement(
            paragraphstylerange_container,
            "ParagraphStyleRange",
            attrib=self.merge_attributes(
                self.PARAGRAPHSTYLERANGE_DEFAULTS, self._attributes.get("ParagraphStyleRange", {})
            ),
        )

        # CharacterStyleRange(s) + <Br />
        for li in self._element.xpath(".//li"):
            if paragraphstylerange.find("CharacterStyleRange") is not None:
                etree.SubElement(paragraphstylerange, "Br")
            paragraphstylerange[:] += self._handle_inline_tags(li)

        return story

    @property
    def length(self):
        raise NotImplementedError

    @staticmethod
    def guess_height(story, inner_width):
        list_height = 0
        list_item_length = 0
        list_item_height = 0

        for el in story._etree.xpath(".//ParagraphStyleRange")[0].iterchildren():

            if el.tag == "Br":
                if list_item_height < 20:
                    list_item_height = 20
                list_height += list_item_height
                # next list item
                list_item_length = 0
                list_item_height = 0
            else:
                list_item_length += len(" ".join(etree.XPath(".//text()")(el)).strip())
                list_item_height += list_item_length / inner_width * 70

        return list_height + 10



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import json
import mimetypes
import bson
import bson.errors
import gridfs
import os.path

from eve.io.mongo.media import GridFSMediaStorage

from . import SuperdeskMediaStorage


logger = logging.getLogger(__name__)


def format_id(_id):
    try:
        return bson.ObjectId(_id)
    except bson.errors.InvalidId:
        return _id


class SuperdeskGridFSMediaStorage(SuperdeskMediaStorage, GridFSMediaStorage):
    def get(self, _id, resource=None):
        logger.debug("Getting media file with id= %s" % _id)
        _id = format_id(_id)
        try:
            media_file = self.fs(resource).get(_id)
        except Exception:
            media_file = None
        if media_file and media_file.metadata:
            for k, v in media_file.metadata.items():
                if isinstance(v, str):
                    try:
                        media_file.metadata[k] = json.loads(v)
                    except ValueError:
                        logger.info("Non JSON metadata for file: %s with key: %s and value: %s", _id, k, v)
        return media_file

    def url_for_media(self, media_id, content_type=None):
        """Return url for given media id.

        :param media_id: media id from media_id method
        """
        ext = mimetypes.guess_extension(content_type or "") or ""
        if ext in (".jpe", ".jpeg"):
            ext = ".jpg"
        return self.app.upload_url(str(media_id) + ext)

    def url_for_download(self, media_id, content_type=None):
        """Return url for download.

        :param media_id: media id from media_id method
        """
        return self.app.download_url(str(media_id))

    def fetch_rendition(self, rendition):
        return self.get(rendition.get("media"), "upload")

    def put(self, content, filename=None, content_type=None, metadata=None, resource=None, folder=None, **kwargs):
        """Store content in gridfs.

        :param content: binary stream
        :param filename: unique filename
        :param content_type: mime type
        :param metadata: file metadata
        :param resource: type of resource
        :param str folder: Folder that the file will be stored in
        :return str: The ID that was generated for this object
        """

        # try to determine mimetype on the server
        content_type = self._get_mimetype(content, filename, content_type)

        if "_id" in kwargs:
            kwargs["_id"] = format_id(kwargs["_id"])

        if folder:
            if folder[-1] == "/":
                folder = folder[:-1]

            if filename:
                filename = "{}/{}".format(folder, filename)

        try:
            logger.info("Adding file {} to the GridFS".format(filename))
            return self.fs(resource).put(
                content, content_type=content_type, filename=filename, metadata=metadata, **kwargs
            )
        except gridfs.errors.FileExists:
            logger.info("File exists filename=%s id=%s" % (filename, kwargs["_id"]))

    def fs(self, resource=None):
        resource = resource or "upload"
        driver = self.app.data.mongo
        px = driver.current_mongo_prefix(resource)
        if px not in self._fs:
            self._fs[px] = gridfs.GridFS(driver.pymongo(prefix=px).db)
        return self._fs[px]

    def remove_unreferenced_files(self, existing_files, resource=None):
        """Get the files from Grid FS and compare against existing files and delete the orphans."""
        current_files = self.fs(resource).find({"_id": {"$nin": list(existing_files)}})
        for file_id in (file._id for file in current_files if str(file._id) not in existing_files):
            print("Removing unused file: ", file_id)
            self.delete(file_id)
        print("Image cleaning completed successfully.")

    def find(self, folder=None, upload_date=None, resource=None):
        """Search for files in the GridFS

        Searches for files in the GridFS using a combination of folder name and/or upload date
        comparisons. The upload date comparisons uses the same mongodb BSON comparison operators,
        i.e. `$eq`, `$gt`, `$gte`, `$lt`, `$lte` and `$ne`, and can be combined together.

        :param str folder: Folder name
        :param dict upload_date: Upload date with comparison operator (i.e. $lt, $lte, $gt or $gte)
        :param resource: The resource type to use
        :return list: List of files that matched the provided parameters
        """
        folder_query = {"filename": {"$regex": "^{}/".format(folder)}} if folder else None
        date_query = {"uploadDate": upload_date} if upload_date else None

        if folder and upload_date:
            query = {"$and": [folder_query, date_query]}
        elif folder:
            query = folder_query
        elif date_query:
            query = date_query
        else:
            query = {}

        files = []
        for file in self.fs(resource).find(query):
            try:
                files.append(
                    {
                        "_id": file._id,
                        "filename": file.filename,
                        "upload_date": file.upload_date,
                        "size": file.length,
                        "_etag": file.md5,
                    }
                )
            except AttributeError as e:
                logging.warning("Failed to get file attributes. {}".format(e))
        return files

    def exists(self, id_or_filename, resource):
        _id = format_id(id_or_filename)
        return super().exists(_id)

    def get_by_filename(self, filename):
        _id, _ = os.path.splitext(filename)
        return self.get(_id)



from unittest import mock
from datetime import datetime
from bson import ObjectId
from apps import saved_searches
from apps.saved_searches.saved_searches import encode_filter, decode_filter
from superdesk.tests import TestCase


class SavedSearchesTestCase(TestCase):
    def test_encode_decode_filter(self):
        data = {"foo": "bar"}
        encoded = encode_filter(data)
        self.assertEqual('{"foo": "bar"}', encoded)
        self.assertEqual('{"foo": "bar"}', encode_filter(encoded))
        decoded = decode_filter(encoded)
        self.assertEqual(data, decoded)
        self.assertEqual(data, decode_filter(decoded))

    @mock.patch.object(saved_searches, "send_report_email")
    def test_publish_report(self, send_report_email):
        """Check that publish_report is called correctly"""
        self.app.data.insert(
            "archive",
            [
                {
                    "_id": 1,
                    "guid": "#1",
                    "genre": {"name": "Article (news)", "qcode": "Article"},
                },
                {
                    "_id": 2,
                    "guid": "#2",
                    "genre": {"name": "Feature", "qcode": "Feature"},
                },
            ],
        )
        self.app.data.insert(
            "desks",
            [
                {
                    "_id": ObjectId("6b56e807cc3a2d1626a0507d"),
                    "guid": "#1",
                    "members": [{"user": "7b56e807cc3a2d1626a0507d"}],
                },
            ],
        )
        self.app.data.insert(
            "saved_searches",
            [
                {
                    "_id": "5b7ebf9b0d6f13085d4bf733",
                    "name": "test_saved_search",
                    "description": "test",
                    "is_global": True,
                    "filter": (
                        '{"query": {"spike": "exclude", "repo": "archive,published,ingest,archived",'
                        '"notgenre": "[\\"Article (news)\\"]"}}'
                    ),
                    "_updated": "2018-09-10 16:45:05+00:00",
                    "_created": "2018-08-23 14:07:23+00:00",
                    "user": "5b56e807cc3a2d1626a0507d",
                    "_etag": "33227606e4403642083da966bfddf0fa056c81b1",
                    "subscribers": {
                        "user_subscriptions": [
                            {
                                "user": "5b56e807cc3a2d1626a0507d",
                                "scheduling": "*/5 * * * *",
                                "next_report": datetime.strptime("2018-09-10 16:50:00+00", "%Y-%m-%d %H:%M:%S+%f"),
                                "last_report": datetime.strptime("2018-09-10 16:45:04+00", "%Y-%m-%d %H:%M:%S+%f"),
                            }
                        ],
                        "desk_subscriptions": [
                            {
                                "desk": ObjectId("6b56e807cc3a2d1626a0507d"),
                                "scheduling": "*/5 * * * *",
                                "next_report": datetime.strptime("2018-09-10 16:50:00+00", "%Y-%m-%d %H:%M:%S+%f"),
                                "last_report": datetime.strptime("2018-09-10 16:45:04+00", "%Y-%m-%d %H:%M:%S+%f"),
                            }
                        ],
                    },
                },
            ],
        )

        with self.app.app_context():
            saved_searches.report()

        self.assertTrue(send_report_email.called)
        self.assertEqual(send_report_email.call_count, 2)
        self.assertEqual(send_report_email.call_args_list[0][0][0], "5b56e807cc3a2d1626a0507d")
        self.assertEqual(send_report_email.call_args_list[1][0][0], "7b56e807cc3a2d1626a0507d")

        sent_docs = send_report_email.call_args[0][2]
        self.assertEqual(len(sent_docs), 1)
        self.assertEqual(sent_docs[0]["guid"], "#2")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
Created on May 29, 2014

@author: ioan
"""

import redis
import arrow
import werkzeug
import superdesk
from bson import ObjectId
from celery import Celery
from kombu.serialization import register
from eve.io.mongo import MongoJSONEncoder
from eve.utils import str_to_date
from flask import json, current_app as app
from superdesk.errors import SuperdeskError
from superdesk.logging import logger


celery = Celery(__name__)
TaskBase = celery.Task


def try_cast(v):
    # False and 0 are getting converted to datetime by arrow
    if v is None or isinstance(v, bool) or v == 0:
        return v

    try:
        str_to_date(v)  # try if it matches format
        return arrow.get(v).datetime  # return timezone aware time
    except Exception:
        try:
            return ObjectId(v)
        except Exception:
            return v


def dumps(o):
    with superdesk.app.app_context():
        return MongoJSONEncoder().encode(o)


def loads(s):
    o = json.loads(s)
    with superdesk.app.app_context():
        return serialize(o)


def serialize(o):
    if isinstance(o, list):
        return [serialize(item) for item in o]
    elif isinstance(o, dict):
        if o.get("kwargs") and not isinstance(o["kwargs"], dict):
            o["kwargs"] = json.loads(o["kwargs"])
        return {k: serialize(v) for k, v in o.items()}
    else:
        return try_cast(o)


register("eve/json", dumps, loads, content_type="application/json")


def handle_exception(exc):
    """Log exception to logger."""
    logger.exception(exc)


class AppContextTask(TaskBase):  # type: ignore
    abstract = True
    serializer = "eve/json"
    app_errors = (
        SuperdeskError,
        werkzeug.exceptions.InternalServerError,  # mongo layer err
    )

    def __call__(self, *args, **kwargs):
        with superdesk.app.app_context():
            try:
                return super().__call__(*args, **kwargs)
            except self.app_errors as e:
                handle_exception(e)

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        with superdesk.app.app_context():
            handle_exception(exc)


celery.Task = AppContextTask


def init_celery(app):
    celery.config_from_object(app.config, namespace="CELERY")
    app.celery = celery
    app.redis = __get_redis(app)


def add_subtask_to_progress(task_id):
    return _update_subtask_progress(task_id, total=True)


def finish_subtask_from_progress(task_id):
    return _update_subtask_progress(task_id, current=True)


def finish_task_for_progress(task_id):
    return _update_subtask_progress(task_id, done=True)


def __get_redis(app_ctx):
    """Constructs Redis Client object.

    :return: Redis Client object
    """
    redis_url = app_ctx.config["REDIS_URL"]
    try:
        return redis.from_url(redis_url)
    except ValueError as e:
        logger.warn("Failed to connect to redis using a connection string: {}".format(e))

        # Newer Redis clients will not accept 'amqp' as the scheme
        # Attempt to mock a redis scheme instead
        protocol = redis_url.split("//")[0]
        new_url = redis_url.replace(protocol, "redis:")
        return redis.from_url(new_url)


def update_key(key, flag=False, db=None):

    if db is None:
        db = app.redis

    if flag:
        crt_value = db.incr(key)
    else:
        crt_value = db.get(key)

    if crt_value:
        crt_value = int(crt_value)
    else:
        crt_value = 0

    return crt_value


def _update_subtask_progress(task_id, current=None, total=None, done=None):
    redis_db = redis.from_url(app.config["REDIS_URL"])
    try:
        current_key = "current_%s" % task_id
        total_key = "total_%s" % task_id
        done_key = "done_%s" % task_id

        crt_current = update_key(current_key, current, redis_db)
        crt_total = update_key(total_key, total, redis_db)
        crt_done = update_key(done_key, done, redis_db)

        if crt_done and crt_current == crt_total:
            redis_db.delete(current_key)
            redis_db.delete(crt_total)
            redis_db.delete(done_key)

        return task_id, crt_current, crt_total
    finally:
        redis_db.disconnect()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive.archive import ArchiveResource, ArchiveVersionsResource
from apps.archive_history import ArchiveHistoryResource

from typing import Any
from superdesk.publish.publish_queue import PublishQueueResource
from superdesk.resource import Resource
from superdesk.metadata.item import get_schema
from superdesk.mongo import TEXT_INDEX_OPTIONS


LEGAL_ARCHIVE_NAME = "legal_archive"
LEGAL_ARCHIVE_VERSIONS_NAME = "legal_archive_versions"
LEGAL_ARCHIVE_HISTORY_NAME = "legal_archive_history"
LEGAL_PUBLISH_QUEUE_NAME = "legal_publish_queue"


class LegalResource(Resource):
    resource_methods = ["GET"]
    item_methods = ["GET"]
    privileges = {"GET": LEGAL_ARCHIVE_NAME}
    mongo_prefix = "LEGAL_ARCHIVE"
    schema = get_schema()


class LegalArchiveResource(LegalResource, ArchiveResource):
    endpoint_name = LEGAL_ARCHIVE_NAME
    resource_title = endpoint_name
    schema = get_schema()
    datasource = {"source": LEGAL_ARCHIVE_NAME}
    versioning = True
    mongo_indexes = ArchiveResource.mongo_indexes.copy()  # type: Any
    mongo_indexes.update(
        {
            "text": (
                [
                    ("headline", "text"),
                    ("slugline", "text"),
                    ("description_text", "text"),
                ],
                TEXT_INDEX_OPTIONS,
            ),
        }
    )


class LegalArchiveVersionsResource(LegalResource, ArchiveVersionsResource):
    endpoint_name = LEGAL_ARCHIVE_VERSIONS_NAME
    resource_title = endpoint_name
    schema = get_schema(versioning=True)
    datasource = {"source": LEGAL_ARCHIVE_VERSIONS_NAME, "projection": {"old_version": 0, "last_version": 0}}


class LegalArchiveHistoryResource(LegalResource, ArchiveHistoryResource):
    endpoint_name = LEGAL_ARCHIVE_HISTORY_NAME
    resource_title = endpoint_name
    schema = get_schema()
    schema.update(
        {
            "update": {"type": "dict", "schema": {}},
        }
    )
    datasource = {"source": LEGAL_ARCHIVE_HISTORY_NAME}
    mongo_indexes = {"item_id": ([("item_id", 1)], {"background": True})}


class LegalPublishQueueResource(LegalResource, PublishQueueResource):
    endpoint_name = LEGAL_PUBLISH_QUEUE_NAME
    resource_title = endpoint_name

    item_schema = {"_subscriber_id": Resource.rel("subscribers")}
    item_schema.update(PublishQueueResource.schema)
    schema = item_schema

    datasource = {"source": LEGAL_PUBLISH_QUEUE_NAME}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from superdesk.io import feeding_service_errors
from superdesk.publish import transmitter_errors
from superdesk.utils import ListCursor

logger = logging.getLogger(__name__)


class IOErrorsService(superdesk.Service):
    def get(self, req, lookup):
        """Return all ingest errors."""
        errors = feeding_service_errors
        io_type = getattr(req, "args", {}).get("io_type", "ingest")
        if io_type == "publish":
            errors = transmitter_errors

        source_type = getattr(req, "args", {}).get("source_type")
        if source_type:
            return ListCursor([self.get_errors_by_source_type(source_type, errors)])
        else:
            return ListCursor([self.get_all_errors(errors)])

    def get_errors_by_source_type(self, source_type, errors):
        return {"source_errors": errors[source_type.lower()], "all_errors": self._get_all_errors(errors)}

    def get_all_errors(self, errors):
        return {"all_errors": self._get_all_errors(errors)}

    def _get_all_errors(self, errors):
        all_errors = {}
        for k, v in errors.items():
            all_errors.update(v)

        return all_errors


class IOErrorsResource(superdesk.Resource):
    resource_methods = ["GET"]
    item_methods = []

    schema = {
        "ingest_error": {"type": "string", "required": True},
        "source_type": {"type": "string"},
        "io_type": {"type": "string"},
        "all_errors": {
            "type": "list",
            "readonly": True,
        },
        "source_errors": {
            "type": "list",
            "readonly": True,
        },
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from unittest import mock
from unittest.mock import MagicMock

from apps.auth.errors import CredentialsAuthError
from superdesk.errors import SuperdeskApiError
from superdesk.tests import TestCase
from superdesk import get_resource_service
from apps.ldap.commands import ImportUserProfileFromADCommand


if os.environ.get("LDAP_SERVER", ""):

    def get_mock_connection():
        """Create a mock ldap connection object.

        :return {object}: mock ldap connection object.
        """
        connection = MagicMock()
        connection.search(return_value=True)
        connection.response = [
            {
                "attributes": {
                    "sn": ["Bar"],
                    "givenName": ["Foo"],
                    "displayName": ["Foo Bar"],
                    "ipPhone": "+1234567890",
                    "email": "foo@bar.com",
                }
            }
        ]

        return connection

    @mock.patch("apps.ldap.ldap.Connection", return_value=get_mock_connection())
    class ImportUsersTestCase(TestCase):
        """for testing import user using ldap."""

        def test_import_user_using_command(self, mock_ldap_connection):
            user = {"username": "user", "password": "pwd", "user_to_import": "barf"}
            cmd = ImportUserProfileFromADCommand()

            cmd.run(user["username"], user["password"], user["user_to_import"])
            auth_user = get_resource_service("auth_db").authenticate({"username": "barf", "password": "dummy"})
            self.assertEqual(auth_user["username"], user["user_to_import"])

            cmd.run(user["username"], user["password"], "BARF")
            auth_user2 = get_resource_service("auth_db").authenticate({"username": "barf", "password": "dummy"})
            self.assertEqual(auth_user2["username"], user["user_to_import"])

            self.assertEqual(auth_user2["_id"], auth_user["_id"])

        @mock.patch("apps.ldap.ldap.get_user", return_value={"username": "user"})
        def test_import_profile_for_already_imported_user_raises_exception(self, mock_ldap_connection, mock_get_user):
            with self.assertRaises(SuperdeskApiError) as context:
                service = get_resource_service("import_profile")
                doc = {"username": "user", "password": "pwd", "profile_to_import": "barf"}
                users = service.post([doc])
                self.assertIsNotNone(users)
                self.assertEqual(len(users), 1)
                doc = {"username": "user", "password": "pwd", "profile_to_import": "BARF "}
                service.post([doc])

            ex = context.exception
            self.assertEqual(ex.message, "User already exists in the system.")
            self.assertEqual(ex.status_code, 400)
            self.assertDictEqual(ex.payload, {"profile_to_import": 1})

        @mock.patch("apps.ldap.ldap.get_user", return_value={"username": "user1"})
        def test_import_user_by_not_logged_in_user_raises_exception(self, mock_ldap_connection, mock_get_user):
            with self.assertRaises(SuperdeskApiError) as context:
                service = get_resource_service("import_profile")
                doc = {"username": "user", "password": "pwd", "profile_to_import": "barf"}
                service.post([doc])

            ex = context.exception
            self.assertEqual(ex.message, "Invalid Credentials.")
            self.assertEqual(ex.status_code, 403)
            self.assertDictEqual(ex.payload, {"credentials": 1})

        @mock.patch(
            "apps.ldap.ldap.ADAuth.authenticate_and_fetch_profile",
            side_effect=CredentialsAuthError(credentials={"username": "test"}, error="test"),
        )
        @mock.patch("apps.ldap.ldap.get_user", return_value={"username": "user"})
        def test_user_import_profile_with_invalid_credentials(self, mock_ldap_connection, mock_auth, mock_get_user):
            with self.assertRaises(SuperdeskApiError) as context:
                service = get_resource_service("import_profile")
                doc = {"username": "user", "password": "pwd", "profile_to_import": "barf"}
                service.post([doc])

            ex = context.exception
            self.assertEqual(ex.message, "Invalid Credentials.")
            self.assertEqual(ex.status_code, 403)
            self.assertDictEqual(ex.payload, {"credentials": 1})
